/**
Busca l'identificador que li correspon al nom de l'acci. 
@param  p_nom_accio Nom de l'acci que correspon amb el procediment creat
@return number Identificador que correspon amb l'acci. En cas que no existeixi
                aquesta acci retorna null.
**/
create or replace function cercaLogAccioNomId(
    p_nom_accio in log_accions_llista.nom_accio%type
) return number as
    p_id number;
begin
    -- Busquem l'acci a la taula
    select id into p_id
    from log_accions_llista
    where nom_accio = p_nom_accio;

    return p_id;
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix la universitat a la taula.
@param  p_id Identificador de la universitat
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixUniversitat(
    p_id in universitat.id%type
) return number as
    check_id number;
begin
    -- Busquem l'acci a la taula
    select id into check_id
    from universitat
    where id = p_id;

    return p_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix el ttol universitari
@param  p_id Identificador del ttol universitari
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTitulacio(
    p_id in titulacio.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from titulacio
    where id = p_id;

    return p_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix el professor universitari
@param  p_nif Nif del professor universitari
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixProfessor(
    p_nif in professor.nif%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
    select nif into check_nif
    from professor
    where nif = p_nif;

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix el professor a la universitat que te una oferta de treball
creada per una empresa
@param  p_nif Nif del professor universitari
@param  p_oferta_treball_id identificador de la oferta de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixProfessorAUnivesitatOfertaTreball(
    p_nif in professor.nif%type,
    p_oferta_treball_id in oferta_treball.id%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix el professor
    select nif into check_nif
    from professor
    where nif = p_nif and universitat_id in(
        select universitat_id
        from oferta_treball
        where id = p_oferta_treball_id
    );

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un estudiant universitari
@param  p_nif Nif de l'estudiant universitari
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixEstudiant(
    p_nif in estudiant.nif%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
    select nif into check_nif
    from estudiant
    where nif = p_nif;

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si l'estudiant universitari seleccionat es el mateix que s'ha inscrit
a la oferta de treball
@param  p_nif Nif de l'estudiant universitari
@param p_oferta_treball_id identificador de l'ofertar de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function esEstudiantSeleccionatAOfertaTreball(
    p_nif in estudiant.nif%type,
    p_oferta_treball_id in oferta_treball.id%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
    select nif into check_nif
    from estudiant
    where nif = p_nif
        and nif in (
            select estudiant_nif
            from candidat
            where oferta_treball_id = p_oferta_treball_id
                and es_apte = 1
        );

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si l'estudiant universitari esta matriculat a la universitat
on s'ha ofertat aquesta oferta de treball
@param  p_nif Nif de l'estudiant universitari
@param p_oferta_treball_id identificador de l'ofertar de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixEstudiantAUniversitatOfertaTreball(
    p_nif in estudiant.nif%type,
    p_oferta_treball_id in oferta_treball.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into  check_id
    from oferta_treball
    where id = p_oferta_treball_id and universitat_id in(
        select universitat_id
        from estudiant
        where nif = p_nif
    );

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si un ttol universitari s'imparteix a una universitat
@param  p_universitat_id identificador de la universitat
@param p_titulacio_id identificador de la titulaci universitria
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTitulacioAUniversitat(
    p_universitat_id in universitat.id%type,
	p_titulacio_id in titulacio.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix
	select universitat_id into check_id
	from universitat_titulacio
	where universitat_id = p_universitat_id and titulacio_id = p_titulacio_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix una companyia
@param  p_cif Cif de la companyia
@return char Si existeix retorna el cif, si no existeix retorna null
**/
create or replace function existeixCompanyia(
    p_cif in companyia.cif%type
) return char as
    check_cif char(9);
begin
     -- Cerquem si existeix 
    select cif into check_cif
    from companyia
    where cif = p_cif;

    return check_cif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix una companyia i esta acceptada pel CIC
@param  p_cif Cif de la companyia
@return char Si existeix retorna el cif, si no existeix retorna null
**/
create or replace function existeixCompanyiaYEstaAcceptada(
    p_cif in companyia.cif%type
) return char as
    check_cif char(9);
begin
     -- Cerquem si existeix 
    select cif into check_cif
    from companyia
    where cif = p_cif and es_acceptada = 1;

    return check_cif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un departament
@param  p_id id del departament
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixDepartament(
    p_id in departament.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from departament
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un treballador
@param  p_nif Nif del treballador 
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTreballador(
    p_nif in treballador.nif%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
    select nif into check_nif
    from treballador
    where nif = p_nif;

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un treballador a la companyia que ha fet la oferta de 
treball
@param  p_nif Nif del treballador 
@param p_oferta_treball_id identificador de la oferta de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTreballadorACompanyiaOfertaTreball(
    p_nif in treballador.nif%type,
    p_oferta_treball_id in oferta_treball.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from oferta_treball
    where id = p_oferta_treball_id and companyia_cif in (
        select companyia_cif
        from treballador
        where nif = p_nif
    );

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un treballador a la companyia que ha ofert la oferta de 
treball i el candidat s'ha apuntat
@param  p_nif Nif del treballador 
@param p_candidat_id identificador de la candidatura
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTreballadorACompanyiaCandidat(
    p_nif in treballador.nif%type,
    p_candidat_id in candidat.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
   select candidat.id into check_id
    from oferta_treball, candidat
    where  candidat.id = p_candidat_id 
            and candidat.oferta_treball_id = oferta_treball.id
            and companyia_cif in(
                select companyia_cif
                from treballador
                where nif = p_nif
            );

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si un treballador te un perms concret
@param  p_nif Nif del treballador 
@param p_permis perms del treballador per comprovar
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function comprovaPermisTreballador(
    p_nif in treballador.nif%type,
    p_permis in treballador.permis%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
    select treballador.nif into check_nif
    from treballador
    where treballador.nif = p_nif
        and bitand(permis, p_permis ) = p_permis;
    
    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix la oferta de treball
@param  p_id id de la oferta de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixOfertaTreball(
    p_id in oferta_treball.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from oferta_treball
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si l'estudiant cursa la titulaci requerida per l'oferta de treball
@param  p_nif Nif de l'estudiant
@param  p_oferta_treball_id id de la oferta de treball
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function cursaEstudiantTitulacioOfertaTreball(
   p_nif in estudiant.nif%type,
   p_oferta_treball_id in oferta_treball.id%type
) return char as
    check_nif char(9);
begin
     -- Cerquem si existeix 
	select nif into check_nif
    from estudiant 
    where nif = p_nif
	and titulacio_id IN (
			select titulacio_id
			from oferta_treball
			where id = p_oferta_treball_id
	);

    return check_nif;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix una candidatura d'un estudiant
@param  p_id id de la candidatura
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixCandidatura(
    p_id in candidat.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from candidat
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si la selecci del candidat a l'oferta de treball ha finalitzat 
amb xit. 
@param  p_id id de la candidatura
@return number Si ha finalitzat retorna 1, en cas contrari retorna 0
**/
create or replace function comprovaCandidaturaFinalitzada(
    p_id in candidat.id%type
) return boolean as
    check_id number;
begin

     -- Cerquem si existeix 
    select id into check_id
    from oferta_treball
    where  id IN (
			select oferta_treball_id
			from candidat
			where id = p_id
		) and process_finalitzat = 1;

    return true;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return false;
end;
/

/**
Comprova si la selecci del candidat de l'oferta de treball ha finalitzat. 
En cas que hagi finalitzat vol dir que ja s'ha seleccionat un estudiant.
@param  p_id id de la candidatura
@return number Si ha finalitzat retorna 1, en cas contrari retorna 0
**/
create or replace function finalitzatOfertaTreball(
    p_id in oferta_treball.id%type
) return boolean as
    check_id number;
begin

     -- Cerquem si existeix 
    select id into check_id
    from oferta_treball
    where id = p_id 
        and process_finalitzat = 1;

    return true;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return false;
end;
/

/**
Comprova si la selecci del candidat de l'oferta de treball ha finalitzat. 
En cas que hagi finalitzat vol dir que ja s'ha seleccionat un estudiant.
@param  p_id id de la candidatura
@return number Si ha finalitzat retorna 1, en cas contrari retorna 0
**/
create or replace function comprovaEstudiantSeleccionat(
    p_id in candidat.id%type
) return boolean as
    check_total number;
begin

     -- Cerquem si existeix 
    select count(estudiant_nif) as total into check_total
    from candidat
    where estudiant_nif IN (
            select estudiant_nif
            from candidat
            where id = p_id
        ) and es_apte = 1;
    return (check_total > 0);
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return false;
end;
/

/**
Comprova si existeix la pregunta realitzada per la companyia a un estudiant 
@param  p_id id de la candidatura
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixPregunta(
    p_id in pregunta_entrevista.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from pregunta_entrevista
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un contracte de treball
@param  p_id id del contracte
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixContracteTreball(
    p_id in contracte_practiques.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from contracte_practiques
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un tema de la formaci
@param  p_id id del tema
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixTemaFormacio(
    p_id in tema_formacio.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from tema_formacio
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix un informe parcial respecte les practiques de l'alumne
@param  p_id id de l'informe
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixInformeParcial(
    p_id in tema_formacio.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from informe_parcial
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix una inspecci de treball
@param  p_id id de l'informe
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixInspeccioTreball(
    p_id in inspeccio_treball.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from inspeccio_treball
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si existeix el mostreig fet a la inspeccio de treball
@param  p_id id de l'informe
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function existeixMostreigInspeccioTreball(
    p_id in inspeccio_treball_mostreig.id%type
) return number as
    check_id number;
begin
     -- Cerquem si existeix 
    select id into check_id
    from inspeccio_treball_mostreig
    where id = p_id;

    return check_id;
	
-- si no la troba dona excepci
exception
    when no_data_found then
        return null;
end;
/

/**
Comprova si un valor esta entre 0 i 1. Com que no existeix el valor bool,
ho fem comprovant si un numero esta entre els dos valors.
@param  p_id id de l'informe
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function comprovaValorsBooleans(
    p_valor in number
) return boolean as
    
begin

    return (p_valor = 0 or p_valor = 1);
-- en qualsevol error retorna fals
exception
    when others then
        return false;
end;
/




/**** ESTADISTIQUES ***/

/**
Comprova si existeix l'any universitari. Un any universitari comena en setembre
i acaba a l'agost. Per exemple, per la data 22/10/2022, l'any universitari 
correspon amb el 22/23, per contra, per la data 22/05/2022, l'any universitari
correspon amb el 21/22. Si no existeix el crea i retorna l'identificador
@param  p_data data per comprovar l'any universitari
@return number Si existeix retorna l'dentificador, si no existeix retorna null
**/
create or replace function buscaAnyUniversitari(
    p_data in date
) return number as
   p_month number;
   p_any number;
   p_any_universitari char(5);
   p_any_universitari_id number;
begin
    p_month := to_number(to_char(p_data, 'mm'));
    p_any := to_number(to_char(p_data, 'yy'));
    
    if (p_month>8) then
        p_any_universitari := (p_any-1) || '/' || p_any;
    else 
        p_any_universitari := p_any || '/' || (p_any+1);   
    end if;
    
    select id into p_any_universitari_id
    from e_any_universitari
    where any_universitari = p_any_universitari;

    return p_any_universitari_id;
exception
     when no_data_found then
        insert into e_any_universitari(any_universitari)
                values(p_any_universitari)
                returning id into p_any_universitari_id;
     
        return p_any_universitari_id;

    
end;
/

