
/**
Comprova si l'acci a realitzar t identificador. Si no el t el crea
@param p_nom_accio Nom de l'acci
@param p_id OUT Identificador que correspon amb l'acci.
**/
create or replace procedure comprovaLogAccioNom(
    p_nom_accio in log_accions_llista.nom_accio%type,
    p_id out log_accions_llista.id%type
) is
    
begin
    -- Cerquem l'identificador per veure si existeix
    p_id := cercaLogAccioNomId(p_nom_accio);
    
    -- Si no existeix el crea
    if p_id is null then
        insert into log_accions_llista(nom_accio)
                                values(p_nom_accio)
                                returning id into p_id;
    end if;
end comprovaLogAccioNom;

/
/**
Registra les accions a la taula de log. 
@param p_nom_accio Nom de l'acci realitzada
@param p_parametre_entrada JSON amb els paràmetres d'entrada de l'acci.
@param p_sortida Missatge que retorna l'acci duta a terme. Si l'execuci s 
                satisfactoria tindra format 'OK'. Si l'execuci dona error, 
                tindra el format 'ERROR'+'TIPUS D'ERROR'
@param p_id OUT Identificador que correspon amb l'acci.
**/
create or replace procedure afegirLog(
    p_nom_accio in log_accions_llista.nom_accio%type,
    p_parametre_entrada in varchar2,
    p_sortida in log_accions.sortida%type
) is
    nom_accio_id number;
    es_error number;
begin
    -- Cerquem l'identificador de l'acci
    comprovaLogAccioNom(p_nom_accio, nom_accio_id);

    -- afegim el registre a la taula
    insert into log_accions(log_accio_llista_id, data_peticio, sortida, parametre_entrada)
                    values( nom_accio_id, current_timestamp, p_sortida, p_parametre_entrada);
end afegirLog;

/

/**
Afegeix una universitat. 
@param p_nom Nom de la universitat
@param p_te_estudiants si t estudiants per practiques o no
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirUniversitat(
    p_nom IN universitat.nom%type, 
    p_te_estudiants in universitat.te_estudiant_per_practiques%type default 1,
    RSP out varchar2
) IS
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_nom', p_nom);
    params.put('p_te_estudiants', p_te_estudiants);
    
	-- afegim el registre
    insert into universitat (nom, te_estudiant_per_practiques)  
                    values(p_nom, p_te_estudiants) 
                    returning id into id;
    commit;
	
    -- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirUniversitat;

/
/**
Elimina una universitat. 
@param p_id identificador de la universitat a eliminar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarUniversitat(
    p_id IN universitat.id%type,
    RSP out varchar2
) IS
    no_universitat exception;
    
    check_id universitat.id%type;
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
    
    -- Cerquem si existeix la universitat
    if (existeixUniversitat(p_id) is null) then
		raise no_universitat;
    end if;
    
	-- Eliminem la universitat
    delete from universitat where id = p_id;
    commit;
    
    afegirLog($$plsql_unit, params, RSP);
exception
    when no_universitat	then
        RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarUniversitat;

/
/**
Modifica les dades d'una universitat 
@param p_id identificador de la universitat a modificar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/

create or replace procedure modificarUniversitat(
    p_id IN universitat.id%type,
    p_nom in universitat.nom%type,
    p_teEstudiants in universitat.te_estudiant_per_practiques%type default 1,
	RSP out varchar2
) IS
    no_universitat exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_nom' value p_nom, 'p_teEstudiants' value p_teEstudiants);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_id) is null) then
		raise no_universitat;
    end if;
	
	update universitat
	set nom = p_nom, te_estudiant_per_practiques = p_teEstudiants
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_universitat then
        RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarUniversitat;

/
/**
Activa o desactiva el fet que una universitat t estudiants en practiques
@param p_id identificador de la universitat a modificar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure universitatTeEstudiantsPerPractiques(
    p_id IN universitat.id%type,
    p_teEstudiants in universitat.te_estudiant_per_practiques%type,
	RSP out varchar2
) IS
    no_universitat exception;
    no_valors_booleans exception;
    params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_teEstudiants' value p_teEstudiants);
    
    -- Cerquem si els valors del parmetre teEstudiants es vlid
    if(comprovaValorsBooleans(p_teEstudiants) = false) then
        raise no_valors_booleans;
    end if;

	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_id) is null) then
		raise no_universitat;
    end if;

	update universitat
	set te_estudiant_per_practiques = p_teEstudiants
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
  
exception
    when no_valors_booleans then
        RSP := 'ERROR|EL VALOR PER TE_ESTUDIANTS NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params, RSP);
    
    when no_universitat then
        RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end universitatTeEstudiantsPerPractiques;

/

/**
Afegeix un ttol universitari a la taula
@param p_id identificador de la universitat a modificar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirTitulacio(
    p_nom in titulacio.nom%type,
	RSP out varchar2
) IS
	id number;
	params json_object_t;

begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_nom', p_nom);
	
	-- afegim la titulaci	
	insert into titulacio (nom)  
				values(p_nom) 
                returning id into id;
	commit;
	
	-- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirTitulacio;

/
/**
Elimina un ttol universitari a la taula
@param p_id identificador del ttol a eliminar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarTitulacio(
    p_id IN titulacio.id%type,
    RSP out varchar2
) IS
    no_titulacio exception;
	check_id titulacio.id%type;
    params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
    
    -- Cerquem si existeix la titulacio
    if (existeixTitulacio(p_id) is null) then
		raise no_titulacio;
    end if;

	delete from titulacio where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_titulacio then
        RSP := 'ERROR|NO EXISTEIX LA TITULACIO';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarTitulacio;

/
/**
Modifica les dades d'un ttol
@param p_id identificador del ttol a modificar
@param p_nom nom del ttol a modificar
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarTitulacio(
    p_id in titulacio.id%type, 
    p_nom in titulacio.nom%type,
	RSP out varchar2
) IS
    no_titulacio exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
    
    -- Cerquem si existeix la titulacio
    if (existeixTitulacio(p_id) is null) then
		raise no_titulacio;
    end if;
	
	update titulacio
	set nom = p_nom
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_titulacio then
        RSP := 'ERROR|NO EXISTEIX LA TITULACIO';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarTitulacio;

/
/**
Afegeix un ttol universitari a una universitat. 
@param p_universitat_id identificador de la universitat
@param p_titulacio_id identificador del ttol universitari
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirTitulacioAUniversitat(
    p_universitat_id in universitat.id%type,
    p_titulacio_id in titulacio.id%type,
    RSP out varchar2
) IS
    no_universitat exception;
    no_titulacio exception;
	params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_universitat_id' value p_universitat_id, 'p_titulacio_id' value p_titulacio_id);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem si existeix la titulaci
    if (existeixTitulacio(p_titulacio_id) is null) then
		raise no_titulacio;
    end if;
	
	insert into universitat_titulacio (universitat_id, titulacio_id)  
								values(p_universitat_id, p_titulacio_id);
	commit;
  
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_universitat then
		RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
		afegirLog($$plsql_unit, params, RSP);
	
	when no_titulacio then
		RSP := 'ERROR|NO EXISTEIX LA TITULACIO';
		afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirTitulacioAUniversitat;

/
/**
Elimina un ttol universitari a una universitat. 
@param p_universitat_id identificador de la universitat
@param p_titulacio_id identificador del ttol universitari
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarTitulacioAUniversitat(
    p_universitat_id in universitat.id%type,
    p_titulacio_id IN titulacio.id%type,
    RSP out varchar2
) IS
    no_universitat exception;
    params varchar2(2000);

begin
	 -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_universitat_id' value p_universitat_id, 'p_titulacio_id' value p_titulacio_id);
    
    -- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;

	delete from universitat_titulacio where universitat_id = p_universitat_id and titulacio_id = p_titulacio_id;
	commit;
  
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_universitat then
        RSP := 'ERROR|NO EXISTEIX AQUEST TITOL A LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarTitulacioAUniversitat;

/
/**
Afegeix un professor a la taula
@param p_nif Nif del professor
@param p_universitat_id Identificador de la universitat en la que treballa
@param p_nom Nom Nom del professor
@param p_cognoms Cognoms del professor
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirProfessor(
    p_nif IN professor.nif%type, 
    p_universitat_id in universitat.id%type,
    p_nom in professor.nom%type,
    p_cognoms in professor.nom%type,
    RSP out varchar2
) IS
    no_universitat exception;
    si_professor exception;
    params varchar2(2000);

begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_nif' value p_nif, 'p_universitat_id' value p_universitat_id, 'p_nom' value p_nom, 'p_cognoms' value p_cognoms);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem si existeix el professor
	if (existeixProfessor(p_nif) is not null) then
		raise si_professor;
    end if;
	
	
	insert into professor ( nif, universitat_id, nom, cognoms ) 
                    values(p_nif, p_universitat_id, p_nom, p_cognoms);
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_universitat then
		RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
		
	when si_professor then
		RSP := 'ERROR|AQUEST PROFESSOR YA EXISTEIX';
        afegirLog($$plsql_unit, params, RSP);
	
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirProfessor;

/
/**
Elimina un professor a la taula
@param p_nif Nif del professor
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarProfessor(
    p_nif IN professor.nif%type,
    RSP out varchar2
) IS
    no_professor exception;
    params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_nif' value p_nif);
	
	-- Cerquem si existeix el professor
	if (existeixProfessor(p_nif) is null) then
		raise no_professor;
    end if;
	
	delete from professor where nif = p_nif;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_professor then
        RSP := 'ERROR|NO EXISTEIX EL PROFESSOR';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarProfessor;

/
/**
Afegeix un professor a la taula
@param p_nif Nif del professor
@param p_nom Nom Nom del professor
@param p_cognoms Cognoms del professor
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarProfessor(
    p_nif in professor.nif%type, 
    p_nom in professor.nom%type,
    p_cognoms in professor.cognoms%type,
	RSP out varchar2
) IS
    no_professor exception;
	params varchar2(2000);

begin
	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_nif' value p_nif, 'p_nom' value p_nom, 'p_cognoms' value p_cognoms);
	
	-- Cerquem si existeix el professor
	if (existeixProfessor(p_nif) is null) then
		raise no_professor;
    end if;
	
	update professor
	set nom = p_nom, cognoms = p_cognoms
	where nif = p_nif;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_professor then
        RSP := 'ERROR|NO EXISTEIX EL PROFESSOR';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end modificarProfessor;

/
/**
Afegeix un estudiant a la taula
@param p_estudiant_nif Nif de l'estudiant
@param p_universitat_id Universitat en la que estudia
@param p_titulacio_id Ttol universitari que estudia
@param p_nom Nom Nom de l'estudiant
@param p_cognoms Cognoms de l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirEstudiant(
    p_estudiant_nif IN estudiant.nif%type, 
    p_universitat_id in universitat.id%type,
    p_titulacio_id in titulacio.id%type,
    p_nom in estudiant.nom%type,
    p_cognoms in estudiant.cognoms%type,
    RSP out varchar2
) IS
    no_universitat exception;
    si_estudiant exception;
    no_titulacio exception;
    no_titol_universitat exception;
    params varchar2(2000);

begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_estudiant_nif' value p_estudiant_nif, 'p_universitat_id' value p_universitat_id, 'p_titulacio_id' value p_titulacio_id, 'p_nom' value p_nom, 'p_cognoms' value p_cognoms);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is not null) then
		raise si_estudiant;
    end if;
	
	-- Cerquem si existeix la titulaci
    if (existeixTitulacio(p_titulacio_id) is null) then
		raise no_titulacio;
    end if;
	
	-- Cerquem si s'imparteix el ttol a la universitat
	if (existeixTitulacioAUniversitat(p_universitat_id, p_titulacio_id) is null) then
		raise no_titol_universitat;
    end if;
	
	insert into estudiant (nif, universitat_id, titulacio_id, nom, cognoms )  
					values(p_estudiant_nif, p_universitat_id, p_titulacio_id, p_nom, p_cognoms );
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
	
exception
	when no_universitat then
		RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
		
	when si_estudiant then
		RSP := 'ERROR|AQUEST ESTUDIANT YA EXISTEIX';
        afegirLog($$plsql_unit, params, RSP);
		
	when no_titulacio then
		RSP := 'ERROR|NO EXISTEIX EL TITOL UNIVERSITARI';
        afegirLog($$plsql_unit, params, RSP);
	
	when no_titol_universitat then
		RSP := 'ERROR|LA UNIVERSITAT NO IMPARTEIX AQUESTA TITULACIO';
        afegirLog($$plsql_unit, params, RSP);
		
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirEstudiant;

/
/**
Modifica les dades d'un estudiant
@param p_estudiant_nif Nif de l'estudiant
@param p_universitat_id Universitat en la que estudia
@param p_titulacio_id Ttol universitari que estudia
@param p_nom Nom Nom de l'estudiant
@param p_cognoms Cognoms de l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarEstudiant(
    p_estudiant_nif IN estudiant.nif%type, 
    p_universitat_id in universitat.id%type,
    p_titulacio_id in titulacio.id%type,
    p_nom in estudiant.nom%type,
    p_cognoms in estudiant.cognoms%type,
	RSP out varchar2
) IS
    no_universitat exception;
    no_estudiant exception;
    no_titulacio exception;
    no_titol_universitat exception;
	params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_estudiant_nif' value p_estudiant_nif, 'p_universitat_id' value p_universitat_id, 'p_titulacio_id' value p_titulacio_id, 'p_nom' value p_nom, 'p_cognoms' value p_cognoms);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is null) then
		raise no_estudiant;
    end if;
	
	-- Cerquem si existeix la titulaci
    if (existeixTitulacio(p_titulacio_id) is null) then
		raise no_titulacio;
    end if;
	
	-- Cerquem si s'imparteix el ttol a la universitat
	if (existeixTitulacioAUniversitat(p_universitat_id, p_titulacio_id) is null) then
		raise no_titol_universitat;
    end if;
	

	update estudiant 
	set universitat_id = p_universitat_id, 
		titulacio_id = p_titulacio_id, 
		nom = p_nom, 
		cognoms = p_cognoms   
	where nif = p_estudiant_nif;
	commit;
 
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_universitat then
		RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
		
	when no_estudiant then
		RSP := 'ERROR|AQUEST ESTUDIANT NO EXISTEIX';
        afegirLog($$plsql_unit, params, RSP);
		
	when no_titulacio then
		RSP := 'ERROR|NO EXISTEIX EL TITOL UNIVERSITARI';
        afegirLog($$plsql_unit, params, RSP);
		
	when no_titol_universitat then
		RSP := 'ERROR|LA UNIVERSITAT NO IMPARTEIX AQUESTA TITULACIO';
        afegirLog($$plsql_unit, params, RSP);
		
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarEstudiant;

/
/**
Elimina un estudiant
@param p_estudiant_nif Nif de l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarEstudiant(
    p_estudiant_nif IN estudiant.nif%type,
    RSP out varchar2
) IS
    no_estudiant exception;
    params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
	-- Creem el JSON per al registre
	params := JSON_OBJECT('p_estudiant_nif' value p_estudiant_nif);

	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is null) then
		raise no_estudiant;
    end if;

	delete from estudiant where nif = p_estudiant_nif;
	commit;
  
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_estudiant then
        RSP := 'ERROR|NO EXISTEIX L''ESTUDIANT';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarEstudiant;

/

/**
Afegeix una companyia
@param p_cif Cif de la companyia
@param p_nom Nom de la companyia
@param p_data_peticio_inscripcio NULL data de la peticio d'entrada al sistema
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirCompanyia(
    p_cif IN companyia.cif%type, 
    p_nom in companyia.nom%type,
    p_data_peticio_inscripcio in companyia.data_peticio_inscripcio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    si_companyia exception;
    params varchar2(2000);

begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_cif' value p_cif, 'p_nom' value p_nom, 'p_data_peticio_inscripcio' value p_data_peticio_inscripcio);

	-- Cerquem la companyia
	if(existeixCompanyia(p_cif) is not null) then
		raise si_companyia;
	end if;

	insert into companyia (cif, nom, data_peticio_inscripcio )  
					values(p_cif, p_nom, p_data_peticio_inscripcio );
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when si_companyia then
		RSP := 'ERROR|YA EXISTEIX LA COMPANYIA';
        afegirLog($$plsql_unit, params, RSP);
		
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirCompanyia;

/
/**
Elimina una companyia
@param p_cif Cif de la companyia
@param p_nom Nom de la companyia
@param p_data_peticio_inscripcio NULL data de la peticio d'entrada al sistema
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarCompanyia(
    p_cif IN companyia.cif%type,
    RSP out varchar2
) IS
    no_companyia exception;
    params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_cif' value p_cif);

	-- Cerquem la companyia
	if(existeixCompanyia(p_cif) is null) then
		raise no_companyia;
	end if;

	delete from companyia where cif = p_cif;
	commit;
  
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_companyia then
		RSP := 'ERROR|NO EXISTEIX LA COMPANYIA';
        afegirLog($$plsql_unit, params, RSP);
		
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarCompanyia;

/
/**
Modifica una companyia
@param p_cif Cif de la companyia
@param p_nom Nom de la companyia
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarCompanyia(
    p_cif in companyia.cif%type, 
    p_nom in companyia.nom%type,
    RSP out varchar2
) IS
    no_companyia exception;
    params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_cif' value p_cif, 'p_nom' value p_nom);

	-- Cerquem la companyia
	if(existeixCompanyia(p_cif) is null) then
		raise no_companyia;
	end if;
	
	update companyia
	set nom = p_nom
	where cif = p_cif;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_companyia then
		RSP := 'ERROR|NO EXISTEIX LA COMPANYIA';
        afegirLog($$plsql_unit, params, RSP);
		
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarCompanyia;

/
/**
Accepta la solicitud d'entrada de la companyia per part del CIC
@param p_cif Cif de la companyia
@param p_es_acceptada Si la companyia es acceptada o no. Nomes pot tenir els valors 0 si no esta acceptada o 1 si esta acceptada. 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure acceptarCompanyia(
    p_cif in companyia.cif%type,
    p_es_acceptada in companyia.es_acceptada%type,
    p_data_revisio_inscripcio in companyia.data_revisio_inscripcio%type default current_timestamp,
    RSP out varchar2
) IS
    no_valors_booleans exception;
    no_companyia exception;
    params varchar2(2000);

begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_cif' value p_cif, 'p_es_acceptada' value p_es_acceptada, 'p_data_revisio_inscripcio' value p_data_revisio_inscripcio);

    -- Comprovem el valor de esAcceptada
    if(comprovaValorsBooleans(p_es_acceptada) = false) then
        raise no_valors_booleans;
    end if;

	-- Cerquem la companyia
	if(existeixCompanyia(p_cif) is null) then
		raise no_companyia;
	end if;
	
	update companyia 
	set es_acceptada = p_es_acceptada,
		data_revisio_inscripcio = p_data_revisio_inscripcio
	where cif = p_cif ;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_valors_booleans then
        RSP := 'ERROR|EL VALOR PER ES_ACCEPTADA NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params, RSP);
    
	when no_companyia then
		RSP := 'ERROR|NO EXISTEIX LA COMPANYIA';
        afegirLog($$plsql_unit, params, RSP);
	
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end acceptarCompanyia;

/
/**
Afegeix un departament d'una companyia
@param p_nom nom del departament
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirDepartament(
    p_nom in departament.nom%type,
    RSP out varchar2
) IS
    id number;
    params json_object_t;
begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_nom', p_nom);

	insert into departament (nom)  
					values(p_nom)
					returning id into id;
	commit;

  	-- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirDepartament;

/
/**
Modifica un departament
@param p_id identificador del departament
@param p_nom nom del departament
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarDepartament(
    p_id in departament.id%type,
    p_nom in departament.nom%type,
    RSP out varchar2
) IS
    no_departament exception;
    params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_nom' value p_nom );
	
	-- Cerquem el departament
	if(existeixDepartament(p_id) is null) then
		raise no_departament;
	end if;
	
	update departament 
	set nom = p_nom
	where id = p_id;
	commit;
  
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_departament then
		RSP := 'ERROR|NO EXISTEIX EL DEPARTAMENT';
        afegirLog($$plsql_unit, params, RSP);
		
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarDepartament;

/
/**
Elimina un departament
@param p_id identificador del departament
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarDepartament(
    p_id in departament.id%type,
    RSP out varchar2
) IS
    no_departament exception;
    params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id );
	
	-- Cerquem el departament
	if(existeixDepartament(p_id) is null) then
		raise no_departament;
	end if;

	delete from departament where id = p_id;
	commit;

	afegirLog($$plsql_unit, params, RSP);
exception
	when no_departament then
		RSP := 'ERROR|NO EXISTEIX EL DEPARTAMENT';
        afegirLog($$plsql_unit, params, RSP);
		
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarDepartament;

/
/**
Afegeix un treballador
@param p_treballador_nif nif del treballador
@param p_companyia_cif cif de la companyia on treballa
@param p_companyia_departament_id identificador del departament
@param p_permis permisos assignats a aquest treballador
@param p_treballador_nom nom del treballador
@param p_treballador_cognoms cognoms del treballador
@param RSP OUT retorna el resultat de l'execuci del procediment
**/

create or replace procedure afegirTreballador(
    p_treballador_nif IN treballador.nif%type, 
    p_companyia_cif in companyia.cif%type,
    p_companyia_departament_id in departament.id%type,
    p_permis in treballador.permis%type,
    p_treballador_nom in treballador.nom%type,
    p_treballador_cognoms in treballador.cognoms%type,
    RSP out varchar2
) IS
    no_companyia exception;
    no_departament exception;
    no_permis exception;
    params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_treballador_nif' value p_treballador_nif, 'p_companyia_cif' value p_companyia_cif, 'p_companyia_departament_id' value p_companyia_departament_id, 'p_permis' value p_permis, 'p_treballador_nom' value p_treballador_nom, 'p_treballador_cognoms' value p_treballador_cognoms );
	
	-- Cerquem la companyia
	if(existeixCompanyia(p_companyia_cif) is null) then
		raise no_companyia;
	end if;
	
	-- Cerquem el departament
	if(existeixDepartament(p_companyia_departament_id) is null) then
		raise no_departament;
	end if;
	
	--comprovem el permis
	if((p_permis = 1 or p_permis = 2 or p_permis = 3) = false) then
		raise no_permis;
	end if;

	insert into treballador (nif, companyia_cif, departament_id, permis, nom, cognoms )  
					values(p_treballador_nif, p_companyia_cif, p_companyia_departament_id, p_permis, p_treballador_nom, p_treballador_cognoms );
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_companyia then
		RSP := 'ERROR|NO EXISTEIX LA COMPANYIA';
        afegirLog($$plsql_unit, params, RSP);
		
	when no_departament then
		RSP := 'ERROR|NO EXISTEIX EL DEPARTAMENT';
        afegirLog($$plsql_unit, params, RSP);
	
	when no_permis then
		RSP := 'ERROR|NO ES PODEN ASIGNAR AQUESTS PERMISOS';
        afegirLog($$plsql_unit, params, RSP);
	
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirTreballador;

/
/**
Elimina un treballador a la taula
@param p_nif Nif del professor
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarTreballador(
    p_nif IN treballador.nif%type,
    RSP out varchar2
) IS
    no_treballador exception;
    params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_nif' value p_nif);
	
	-- Cerquem el treballador
	if(existeixTreballador(p_nif) is null) then
		raise no_treballador;
	end if;	
	
	delete from treballador where nif = p_nif;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_treballador then
		RSP := 'ERROR|NO EXISTEIX EL TREBALLADOR';
        afegirLog($$plsql_unit, params, RSP);
		
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
		
end eliminarTreballador;

/
/**
Modifica un treballador
@param p_nif nif del treballador
@param p_nom nom del treballador
@param p_cognoms cognoms del treballador
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarTreballador(
    p_nif IN treballador.nif%type,
    p_nom in companyia.nom%type,
    p_cognoms in companyia.nom%type,
    RSP out varchar2
) IS
    no_treballador exception;
    params varchar2(2000);

begin

	-- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_nif' value p_nif, 'p_nom' value p_nom, 'p_cognoms' value p_cognoms );

	-- Cerquem el treballador
	if(existeixTreballador(p_nif) is null) then
		raise no_treballador;
	end if;	
	
	update treballador
	set nom = p_nom, cognoms = p_cognoms
	where nif = p_nif;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
	
exception
	when no_treballador then
		RSP := 'ERROR|NO EXISTEIX EL TREBALLADOR';
        afegirLog($$plsql_unit, params, RSP);
	
    when others then
        RSP := concat('ERROR|', SQLERRM);
end modificarTreballador;

/
/**
Afegeix una oferta de treball
@param p_universitat_id nif del treballador
@param p_companyia_cif nom del treballador
@param p_titulacio_id cognoms del treballador
@param p_oferta_nom cognoms del treballador
@param p_oferta_descripcio cognoms del treballador
@param p_data_creacio cognoms del treballador
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirOfertaTreball(
    p_universitat_id IN universitat.id%type, 
    p_companyia_cif in companyia.cif%type,
    p_titulacio_id in titulacio.id%type,
    p_oferta_nom in oferta_treball.nom%type,
    p_oferta_descripcio in oferta_treball.descripcio_requisits%type,
    p_data_creacio in oferta_treball.data_creacio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_universitat exception;
    no_companyia exception;
    no_titulacio exception;
    no_titol_universitat exception;
    
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_universitat_id', p_universitat_id);
    params.put('p_companyia_cif', p_companyia_cif);
	params.put('p_titulacio_id', p_titulacio_id);
	params.put('p_oferta_nom', p_oferta_nom);
	params.put('p_oferta_descripcio', p_oferta_descripcio);
	params.put('p_data_creacio', p_data_creacio);
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem la companyia y si esta acceptada pel CIC
	if(existeixCompanyiaYEstaAcceptada(p_companyia_cif) is null) then
		raise no_companyia;
	end if;

	-- Cerquem si existeix la titulacio
    if (existeixTitulacio(p_titulacio_id) is null) then
		raise no_titulacio;
    end if;
	
	-- Cerquem si s'imparteix el ttol a la universitat
	if (existeixTitulacioAUniversitat(p_universitat_id, p_titulacio_id) is null) then
		raise no_titol_universitat;
    end if;


	insert into oferta_treball (universitat_id, companyia_cif, titulacio_id, nom, descripcio_requisits, data_creacio, process_finalitzat)  
						values(p_universitat_id, p_companyia_cif, p_titulacio_id, p_oferta_nom, p_oferta_descripcio, p_data_creacio, 0 ) 
						returning id into id;
                        
	commit;
  
	-- afegim l'identificador al JSON
	params.put('id', id);
	afegirLog($$plsql_unit, params.to_string, RSP);
exception
	when no_universitat then
        RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_companyia then
        RSP := 'ERROR|NO EXISTEIX LA COMPANYIA O NO HA ESTAT ACCEPTADA PEL CIC';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_titulacio then
        RSP := 'ERROR|NO EXISTEIX LA TITULACI';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_titol_universitat then
		RSP := 'ERROR|NO EXISTEIX LA TITULACI A LA UNIVERSITAT';
        afegirLog($$plsql_unit, params.to_string, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirOfertaTreball;

/
/**
Modifica una oferta de treball. La modificaci la fa la companyia, per això es passa la companyia com a paràmetre
@param p_id Identificador de la oferta de treball
@param p_universitat_id Identificador de la universitat
@param p_titulacio_id Identificador del ttol universitari
@param p_oferta_nom Ttol de la oferta
@param p_oferta_descripcio Descripci de la oferta
@param p_data_creacio data de creaci de la oferta
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarOfertaTreball(
    p_id in oferta_treball.id%type,
    p_universitat_id IN universitat.id%type, 
    p_titulacio_id in titulacio.id%type,
    p_oferta_nom in oferta_treball.nom%type,
    p_oferta_descripcio in oferta_treball.descripcio_requisits%type,
    p_data_creacio in oferta_treball.data_creacio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_oferta_treball exception;
    no_universitat exception;
    no_titulacio exception;
    no_titol_universitat exception;
    no_companyia exception;
    
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id, 'p_universitat_id' value p_universitat_id, 'p_titulacio_id' value p_titulacio_id, 'p_oferta_nom' value p_oferta_nom, 'p_oferta_descripcio' value p_oferta_descripcio, 'p_data_creacio' value p_data_creacio);
	
	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_id) is null) then
		raise no_oferta_treball;
    end if;
	
	-- Cerquem si existeix la universitat
    if (existeixUniversitat(p_universitat_id) is null) then
		raise no_universitat;
    end if;
	
	-- Cerquem si existeix la titulacio
    if (existeixTitulacio(p_titulacio_id) is null) then
		raise no_titulacio;
    end if;
	
	-- Cerquem si s'imparteix el ttol a la universitat
	if (existeixTitulacioAUniversitat(p_universitat_id, p_titulacio_id) is null) then
		raise no_titol_universitat;
    end if;

	update oferta_treball 
	set universitat_id = p_universitat_id, 
		titulacio_id = p_titulacio_id, 
		nom = p_oferta_nom, 
		descripcio_requisits = p_oferta_descripcio
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
	when no_universitat then
        RSP := 'ERROR|NO EXISTEIX LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
	when no_companyia then
        RSP := 'ERROR|NO EXISTEIX LA COMPANYIA O NO HA ESTAT ACCEPTADA PEL CIC';
        afegirLog($$plsql_unit, params, RSP);
	when no_titulacio then
        RSP := 'ERROR|NO EXISTEIX LA TITULACI';
        afegirLog($$plsql_unit, params, RSP);
	when no_titol_universitat then
		RSP := 'ERROR|NO EXISTEIX LA TITULACI A LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarOfertaTreball;

/
/**
Elimina una oferta de treball
@param p_universitat_id nif del treballador
@param p_titulacio_id cognoms del treballador
@param p_oferta_nom cognoms del treballador
@param p_oferta_descripcio cognoms del treballador
@param p_data_creacio cognoms del treballador
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarOfertaTreball(
    p_id in oferta_treball.id%type,
    RSP out varchar2
) IS
    no_oferta_treball exception;

    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id);
	
	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_id) is null) then
		raise no_oferta_treball;
    end if;

	delete from oferta_treball where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
  
end eliminarOfertaTreball;

/
/**
Afegir una candidatura a una oferta de treball
@param p_estudiant_nif nif de l'estudiant
@param p_oferta_treball_id identificador de la oferta de treball
@param p_requisits_empresa contestaci de requisits demanats per l'empresa
@param p_data_inscripcio data de la inscripci de l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirCandidatura(
    p_estudiant_nif IN estudiant.nif%type, 
    p_oferta_treball_id in oferta_treball.id%type,
    p_requisits_empresa in candidat.requisits_empresa%type,
    p_data_inscripcio in candidat.data_inscripcio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_estudiant exception;
    no_oferta_treball exception;    
    no_estudiant_titulacio_oferta_treball exception;  
    no_estudiant_universitat_oferta_treball exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_estudiant_nif', p_estudiant_nif);
    params.put('p_oferta_treball_id', p_oferta_treball_id);
	params.put('p_requisits_empresa', p_requisits_empresa);
	params.put('p_data_inscripcio', p_data_inscripcio);
	
	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is null) then
		raise no_estudiant;
    end if;
	
	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_oferta_treball_id) is null) then
		raise no_oferta_treball;
    end if;
	
	-- Cerquem Si l'estudiant cursa els estudis necessaris per afegir la candidatura
	if(cursaEstudiantTitulacioOfertaTreball(p_estudiant_nif, p_oferta_treball_id) is null) then
		raise no_estudiant_titulacio_oferta_treball;
	end if;
    
    -- Cerquem Si l'estudiant cursa a la universitat correcta
	if(existeixEstudiantAUniversitatOfertaTreball(p_estudiant_nif, p_oferta_treball_id) is null) then
		raise no_estudiant_universitat_oferta_treball;
	end if;
    
	
	insert into candidat (estudiant_nif, oferta_treball_id, requisits_empresa, data_inscripcio )  
						values(p_estudiant_nif, p_oferta_treball_id, p_requisits_empresa, p_data_inscripcio )
						returning id into id;
	commit;
	
	-- afegim l'identificador al JSON
	params.put('id', id);
	
	afegirLog($$plsql_unit, params.to_string, RSP);
exception
    when no_estudiant then
        RSP := 'ERROR|NO EXISTEIX L''ESTUDIANT';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_estudiant_titulacio_oferta_treball then
		RSP := 'ERROR|L''ESTUDIANT NO CURSA EL TTOL UNIVERSITARI NECESSARI';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_estudiant_universitat_oferta_treball then
        RSP := 'ERROR|L''ESTUDIANT NO ESTUDIA A LA UNIVERSITAT DE L''OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);	
		
end afegirCandidatura;

/
/**
Modifica una candidatura a una oferta de treball. Com que el modifica un estudiant, no cal passar aquest paràmetre
@param p_id identificador de la candidatura
@param p_oferta_treball_id identificador de la oferta de treball
@param p_requisits_empresa contestaci de requisits demanats per l'empresa
@param p_data_inscripcio data de la inscripci de l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarCandidatura(
    p_id in candidat.id%type,
    p_oferta_treball_id in oferta_treball.id%type,
    p_requisits_empresa in candidat.requisits_empresa%type,
    p_data_inscripcio in candidat.data_inscripcio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_candidat exception;
    no_oferta_treball exception;
    
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id, 'p_oferta_treball_id' value p_oferta_treball_id, 'p_requisits_empresa' value p_requisits_empresa, 'p_data_inscripcio' value p_data_inscripcio);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_id) is null) then
		raise no_candidat;
    end if;
	
	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_id) is null) then
		raise no_oferta_treball;
    end if;
	
	update candidat
	set oferta_treball_id = p_oferta_treball_id, 
		requisits_empresa = p_requisits_empresa, 
		data_inscripcio = p_data_inscripcio
	where id = p_id;
	commit;
  
  afegirLog($$plsql_unit, params, RSP);
exception
	when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params, RSP);
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);

	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end modificarCandidatura;

/
/**
Elimina una candidatura
@param p_id identificador de la candidatura
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarCandidatura(
    p_id in candidat.id%type,
    RSP out varchar2
) IS
    no_candidat exception;

    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_id) is null) then
		raise no_candidat;
    end if;

	delete from candidat where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params, RSP);
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end eliminarCandidatura;

/
/**
Envia la candidatura a la companyia per la seva avaluacio
@param p_id identificador de la candidatura
@param p_data_enviada_empresa Data de quan s'ha enviat a l'empresa
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure enviarCandidaturaACompanyia(
    p_id IN candidat.id%type,
    p_data_enviada_empresa in candidat.data_enviada_empresa%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_candidat exception;
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id, 'p_data_enviada_empresa' value p_data_enviada_empresa);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_id) is null) then
		raise no_candidat;
    end if;
	
	update candidat
	set data_enviada_empresa = p_data_enviada_empresa
	where id = p_id;
    
    commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params, RSP);
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);		
	
end enviarCandidaturaACompanyia;

/
/**
Validaci de la candidatura per part d'un treballador de la companyia. Un cop validada, passa al procès de selecci
@param p_id identificador de la candidatura
@param p_treballador_nif treballador que valida la candidatura per la seva selecci
@param p_data_enviada_empresa Data de quan s'ha enviat a l'empresa
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure validarCandidaturaPerCompanyia(
    p_id IN candidat.id%type,
    p_treballador_nif IN treballador.nif%type,
    p_data_validat in candidat.data_validat%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_candidat exception;
    no_treballador exception;
    no_treballador_permis exception;
    no_treballador_a_companyia exception;
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id, 'p_treballador_nif' value p_treballador_nif, 'p_data_validat' value p_data_validat);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_id) is null) then
		raise no_candidat;
    end if;
	
	-- Cerquem el treballador
	if(existeixTreballador(p_treballador_nif) is null) then
		raise no_treballador;
	end if;
    
    -- Cerquem el treballador a la companyia que fa la oferta de treball
	if(existeixTreballadorACompanyiaCandidat(p_treballador_nif, p_id) is null) then
		raise no_treballador_a_companyia;
	end if;
    
     -- Cerquem el treballador a la companyia que fa la oferta de treball
	if(comprovaPermisTreballador(p_treballador_nif, 1) is null) then
		raise no_treballador_permis;
	end if;
    
	update candidat
	set data_validat = p_data_validat, validat_per = p_treballador_nif
	where id = p_id;
  
    commit;
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_treballador then
        RSP := 'ERROR|NO EXISTEIX EL TREBALLADOR';
        afegirLog($$plsql_unit, params, RSP);
	when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params, RSP);
    when no_treballador_a_companyia then
        RSP := 'ERROR|NO EXISTEIX TREBALLADOR A LA COMPANYIA QUE HA FET LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
    when no_treballador_permis then
        RSP := 'ERROR|AQUEST TREBALLADOR NO TE PERMS PER VAL.LIDAR';
        afegirLog($$plsql_unit, params, RSP);
        
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end validarCandidaturaPerCompanyia;

/
/**
Selecci de la candidatura per part d'un treballador de la companyia.
@param p_id identificador de la candidatura
@param p_treballador_nif treballador que valida la candidatura per la seva selecci
@param p_es_apte Si el candidat es apte o no
@param p_data_seleccionat Data de quan s'ha seleccionat
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure seleccionarCandidaturaPerCompanyia(
    p_id IN candidat.id%type,
    p_treballador_nif IN treballador.nif%type,
    p_es_apte IN candidat.es_apte%type,
    p_data_seleccionat in candidat.data_seleccionat%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_candidat exception;
    no_treballador exception;
    no_valors_apte exception;
    estudiant_ja_seleccionat exception;
    oferta_treball_finalitzada exception;
    no_treballador_a_companyia exception;
    no_treballador_permis exception;
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT( 'p_id' value p_id, 'p_treballador_nif' value p_treballador_nif, 'p_es_apte' value p_es_apte, 'p_data_seleccionat' value p_data_seleccionat);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_id) is null) then
		raise no_candidat;
    end if;
	
	-- Cerquem el treballador
	if(existeixTreballador(p_treballador_nif) is null) then
		raise no_treballador;
	end if;
	
	-- Comprovem els valors de acceptaci de la companyia
	if(comprovaValorsBooleans(p_es_apte) = false) then
		raise no_valors_apte;
	end if;
    
    -- Cerquem el treballador a la companyia que fa la oferta de treball
	if(existeixTreballadorACompanyiaCandidat(p_treballador_nif, p_id) is null) then
		raise no_treballador_a_companyia;
	end if;
    
     -- Cerquem el treballador a la companyia que fa la oferta de treball
	if(comprovaPermisTreballador(p_treballador_nif, 2) is null) then
		raise no_treballador_permis;
	end if;
	
	if p_es_apte = 1 then
	
		-- Cerquem si l'oferta de treball encara esta activa
		if (comprovaCandidaturaFinalitzada(p_id) = true) then
			raise oferta_treball_finalitzada;
		end if;
        
        -- Cerquem si el candidat ja ha estat seleccionat
		if (comprovaEstudiantSeleccionat(p_id) = true) then
			raise estudiant_ja_seleccionat;
		end if;
	
		update oferta_treball
		set process_finalitzat = 1
		where id IN (
			select oferta_treball_id
			from candidat
			where id = p_id
		);
	end if;
	
	update candidat
	set data_seleccionat = p_data_seleccionat, seleccionat_per = p_treballador_nif, es_apte = p_es_apte
	where id = p_id;
  
	commit;
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_valors_apte then
		RSP := 'ERROR|EL VALOR PER ES_APTE NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params, RSP);
	when oferta_treball_finalitzada then
        RSP := 'ERROR|EL PROCS DE SELECCI JA HA FINALITZAT';
        afegirLog($$plsql_unit, params, RSP);
	when no_treballador then
        RSP := 'ERROR|NO EXISTEIX EL TREBALLADOR';
        afegirLog($$plsql_unit, params, RSP);
	when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params, RSP);
    when estudiant_ja_seleccionat then
        RSP := 'ERROR|L''ESTUDIANT JA ESTA SELECCIONAT';
        afegirLog($$plsql_unit, params, RSP);
    when no_treballador_a_companyia then
        RSP := 'ERROR|NO EXISTEIX TREBALLADOR A LA COMPANYIA QUE HA FET LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
    when no_treballador_permis then
        RSP := 'ERROR|AQUEST TREBALLADOR NO TE PERMS PER VAL.LIDAR';
        afegirLog($$plsql_unit, params, RSP);
	when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end seleccionarCandidaturaPerCompanyia;

/
/**
Afegeix una pregunta feta per la companyia a l'estudiant en proces de selecci
@param p_candidatura_id identificador de la candidatura
@param p_pregunta Pregunta feta per la companyia a l'estudiant
@param p_data_pregunta Data de realitzacio de la pregunta
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirPreguntaEntrevista(
    p_candidatura_id IN candidat.id%type, 
    p_pregunta in pregunta_entrevista.pregunta%type,
    p_data_pregunta in pregunta_entrevista.data_pregunta%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_candidat exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_candidatura_id', p_candidatura_id);
    params.put('p_pregunta', p_pregunta);
	params.put('p_data_pregunta', p_data_pregunta);
	
	-- Cerquem si existeix la candidatura
	if (existeixCandidatura(p_candidatura_id) is null) then
		raise no_candidat;
    end if;
	
	insert into pregunta_entrevista (candidat_id, pregunta, data_pregunta )  
					values(p_candidatura_id, p_pregunta, p_data_pregunta )
					returning id into id;
	commit;
	
	-- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
    when no_candidat then
        RSP := 'ERROR|NO EXISTEIX LA CANDIDATURA';
        afegirLog($$plsql_unit, params.to_string, RSP); 
    
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirPreguntaEntrevista;

/
/**
Modifica una pregunta feta per la companyia a l'estudiant en proces de selecci
@param p_id identificador de la candidatura
@param p_pregunta Pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarPreguntaEntrevista(
    p_id IN pregunta_entrevista.id%type, 
    p_pregunta in pregunta_entrevista.pregunta%type,
    RSP out varchar2
) IS
    no_pregunta exception;

    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
	
	-- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_pregunta' value p_pregunta);
    
    -- Cerquem si existeix la pregunta
	if (existeixPregunta(p_id)  is null) then
		raise no_pregunta;
    end if;
	
	update pregunta_entrevista
	set pregunta = p_pregunta
	where id = p_id;
	commit;
	
   afegirLog($$plsql_unit, params, RSP);
exception
    when no_pregunta then
        RSP := 'ERROR|NO EXISTEIX LA PREGUNTA';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
		
end modificarPreguntaEntrevista;

/
/**
Modifica una pregunta feta per la companyia a l'estudiant en proces de selecci
@param p_id identificador de la candidatura
@param p_pregunta Pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarPreguntaEntrevista(
    p_id IN pregunta_entrevista.id%type,
    RSP out varchar2
) IS
    no_pregunta exception;
    
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
	
	-- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
    
    -- Cerquem si existeix la pregunta
	if (existeixPregunta(p_id)  is null) then
		raise no_pregunta;
    end if;
	
	delete from pregunta_entrevista  where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_pregunta then
        RSP := 'ERROR|NO EXISTEIX LA PREGUNTA';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
		
end eliminarPreguntaEntrevista;

/
/**
Afegeix una resposta a una pregunta realitzada per la companyia a l'estudiant en proces de selecci
@param p_pregunta_entrevista_id identificador de la pregunta
@param p_resposta Resposta a la pregunta feta per la companyia a l'estudiant
@param p_data_resposta data de la  realitzaci de la resposta 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirRespostaEntrevista(
    p_pregunta_entrevista_id IN pregunta_entrevista.id%type, 
    p_resposta in pregunta_entrevista.resposta%type,
    p_data_resposta in pregunta_entrevista.data_resposta%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_pregunta exception;
    
    params varchar2(2000);
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
	
	-- Creem el JSON per al registre
    params := JSON_OBJECT('p_pregunta_entrevista_id' value p_pregunta_entrevista_id, 'p_resposta' value p_resposta, 'p_data_resposta' value p_data_resposta);
    
    -- Cerquem si existeix la pregunta
	if (existeixPregunta(p_pregunta_entrevista_id)  is null) then
		raise no_pregunta;
    end if;
	
	update pregunta_entrevista
	set resposta = p_resposta, data_resposta = p_data_resposta
	where id = p_pregunta_entrevista_id;
	commit;

	afegirLog($$plsql_unit, params, RSP);
exception
    when no_pregunta then
        RSP := 'ERROR|NO EXISTEIX LA PREGUNTA';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
	
end afegirRespostaEntrevista;

/
/**
Afegeix un contracte realitzat per la companyia a l'estudiant seleccionat 
@param p_oferta_treball_id identificador de la pregunta
@param p_treballador_nif Resposta a la pregunta feta per la companyia a l'estudiant
@param p_estudiant_nif data de la  realitzaci de la resposta 
@param p_data_inici data de la  realitzaci de la resposta 
@param p_data_fi data de la finalitzacio del contracte
@param p_sou sou que rep l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirContracte(
    p_oferta_treball_id IN oferta_treball.id%type, 
    p_treballador_nif in treballador.nif%type,
    p_estudiant_nif in estudiant.nif%type,
    p_professor_nif in professor.nif%type,
    p_data_inici in contracte_practiques.data_inici%type,
    p_data_fi in contracte_practiques.data_fi%type,
    p_sou in contracte_practiques.sou%type,
    RSP out varchar2
) IS
    no_oferta_treball exception;
    no_estudiant exception;
    oferta_treball_no_finalitzada exception;
    no_professor exception;
    no_treballador exception;
    no_companyia exception;
    no_estudiant_seleccionat exception;
    no_professor_universitat exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_oferta_treball_id', p_oferta_treball_id);
    params.put('p_treballador_nif', p_treballador_nif);
	params.put('p_professor_nif', p_professor_nif);
	params.put('p_estudiant_nif', p_estudiant_nif);
	params.put('p_data_inici', p_data_inici);
	params.put('p_data_fi', p_data_fi);
	params.put('p_sou', p_sou);
	
	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_oferta_treball_id) is null) then
		raise no_oferta_treball;
    end if;
	
	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is null) then
		raise no_estudiant;
    end if;
	
	-- Cerquem si l'oferta de treball encara esta activa
	if (finalitzatOfertaTreball(p_oferta_treball_id) = false) then
		raise oferta_treball_no_finalitzada;
    end if;
	
	-- Cerquem si existeix el professor
	if (existeixProfessor(p_professor_nif) is null) then
		raise no_professor;
    end if;
	
	-- Cerquem el treballador
	if(existeixTreballador(p_treballador_nif) is null) then
		raise no_treballador;
	end if;
    
    -- Cerquem el treballador
	if(esEstudiantSeleccionatAOfertaTreball(p_estudiant_nif, p_oferta_treball_id) is null) then
		raise no_estudiant_seleccionat;
	end if;
    
    -- Cerquem el treballador
	if(existeixProfessorAUnivesitatOfertaTreball(p_professor_nif, p_oferta_treball_id) is null) then
		raise no_professor_universitat;
	end if;
    
    
	insert into contracte_practiques (oferta_treball_id, treballador_nif, estudiant_nif, professor_nif, data_inici, data_fi, sou )  
					values(p_oferta_treball_id, p_treballador_nif, p_estudiant_nif, p_professor_nif, p_data_inici, p_data_fi, p_sou ) 
                    returning id into id;
    commit;
	
	-- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
	when no_professor then
        RSP := 'ERROR|NO EXISTEIX EL PROFESSOR';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when oferta_treball_no_finalitzada then
		RSP := 'ERROR|NO S''HA SELECCIONAT CAP CANDIDAT PER LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_companyia then
        RSP := 'ERROR|NO EXISTEIX LA COMPANYIA O NO HA ESTAT ACCEPTADA PEL CIC';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_estudiant then
		RSP := 'ERROR|AQUEST ESTUDIANT NO EXISTEIX';
        afegirLog($$plsql_unit, params.to_string, RSP);
    when no_estudiant_seleccionat then
		RSP := 'ERROR|AQUEST ESTUDIANT NO ES EL SELECCIONAT A L''OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
    when no_professor_universitat then
		RSP := 'ERROR|AQUEST PROFESSOR NO PERTANY A LA UNIVERSITAT';
        afegirLog($$plsql_unit, params.to_string, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirContracte;

/
/**
Modifica un contracte realitzat per la companyia a l'estudiant seleccionat 
@param p_id 
@param p_oferta_treball_id identificador de la pregunta
@param p_treballador_nif Resposta a la pregunta feta per la companyia a l'estudiant
@param p_estudiant_nif data de la  realitzaci de la resposta 
@param p_data_inici data de la  realitzaci de la resposta 
@param p_data_fi data de la finalitzacio del contracte
@param p_sou sou que rep l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarContracte(
    p_id in contracte_practiques.id%type,
    p_oferta_treball_id IN oferta_treball.id%type, 
    p_treballador_nif in treballador.nif%type,
    p_estudiant_nif in estudiant.nif%type,
    p_professor_nif in professor.nif%type,
    p_data_inici in contracte_practiques.data_inici%type,
    p_data_fi in contracte_practiques.data_fi%type,
    p_sou in contracte_practiques.sou%type,
	RSP out varchar2
) IS
    no_oferta_treball exception;
    no_estudiant exception;
    no_contracte_treball exception;
    oferta_treball_no_finalitzada exception;
    no_professor exception;
    no_treballador exception;
    no_companyia exception;
    no_estudiant_seleccionat exception;
    no_professor_universitat exception;
    
    p_sou_anterior number;
	params varchar2(2000);
begin

    -- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_oferta_treball_id' value p_oferta_treball_id, 'p_treballador_nif' value p_treballador_nif, 'p_estudiant_nif' value p_estudiant_nif, 'p_professor_nif' value p_professor_nif, 'p_data_inici' value p_data_inici, 'p_data_fi' value p_data_fi, 'p_sou' value p_sou);

	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_id)  is null) then
		raise no_contracte_treball;
    end if;

	-- Cerquem si existeix la oferta de treball
	if (existeixOfertaTreball(p_oferta_treball_id) is null) then
		raise no_oferta_treball;
    end if;
	
	-- Cerquem si existeix el l'estudiant
	if (existeixEstudiant(p_estudiant_nif) is null) then
		raise no_estudiant;
    end if;
	
	-- Cerquem si l'oferta de treball encara esta activa
	if (finalitzatOfertaTreball(p_oferta_treball_id) = false) then
		raise oferta_treball_no_finalitzada;
    end if;
	
	-- Cerquem si existeix el professor
	if (existeixProfessor(p_professor_nif) is null) then
		raise no_professor;
    end if;
	
	-- Cerquem el treballador
	if(existeixTreballador(p_treballador_nif) is null) then
		raise no_treballador;
	end if;	
    
    -- Cerquem el treballador
	if(esEstudiantSeleccionatAOfertaTreball(p_estudiant_nif, p_oferta_treball_id) is null) then
		raise no_estudiant_seleccionat;
	end if;
    
    -- Cerquem el treballador
	if(existeixProfessorAUnivesitatOfertaTreball(p_professor_nif, p_oferta_treball_id) is null) then
		raise no_professor_universitat;
	end if;
    
    
    select sou into p_sou_anterior
    from contracte_practiques
    where id = p_id;
    
	update contracte_practiques 
	set oferta_treball_id = p_oferta_treball_id, 
		treballador_nif = p_treballador_nif, 
		estudiant_nif = p_estudiant_nif, 
		professor_nif = p_professor_nif, 
		data_inici = p_data_inici , 
		data_fi = p_data_fi, 
		sou = p_sou
	where id = p_id;
	commit;
	
    afegirLog($$plsql_unit, params, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
	when no_professor then
        RSP := 'ERROR|NO EXISTEIX EL PROFESSOR';
        afegirLog($$plsql_unit, params, RSP);
	when no_oferta_treball then
        RSP := 'ERROR|NO EXISTEIX LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
	when oferta_treball_no_finalitzada then
		RSP := 'ERROR|NO S''HA SELECCIONAT CAP CANDIDAT PER LA OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
	when no_companyia then
        RSP := 'ERROR|NO EXISTEIX LA COMPANYIA O NO HA ESTAT ACCEPTADA PEL CIC';
        afegirLog($$plsql_unit, params, RSP);
	when no_estudiant then
		RSP := 'ERROR|AQUEST ESTUDIANT NO EXISTEIX';
        afegirLog($$plsql_unit, params, RSP);
    when no_estudiant_seleccionat then
		RSP := 'ERROR|AQUEST ESTUDIANT NO ES EL SELECCIONAT A L''OFERTA DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
    when no_professor_universitat then
		RSP := 'ERROR|AQUEST PROFESSOR NO PERTANY A LA UNIVERSITAT';
        afegirLog($$plsql_unit, params, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
		
end modificarContracte;

/
/**
Modifica un contracte realitzat per la companyia a l'estudiant seleccionat 
@param p_id 
@param p_oferta_treball_id identificador de la pregunta
@param p_treballador_nif Resposta a la pregunta feta per la companyia a l'estudiant
@param p_estudiant_nif data de la  realitzaci de la resposta 
@param p_data_inici data de la  realitzaci de la resposta 
@param p_data_fi data de la finalitzacio del contracte
@param p_sou sou que rep l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarContracte(
    p_id in contracte_practiques.id%type,
	RSP out varchar2
) IS
    no_contracte_treball exception;
	params varchar2(2000);
    p_es_apte number;
    p_sou_anterior number;
begin
    -- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);

	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_id)  is null) then
		raise no_contracte_treball;
    end if;
    
	delete from contracte_practiques  where id = p_id;
    
	commit;
    afegirLog($$plsql_unit, params, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
		afegirLog($$plsql_unit, params, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarContracte;

/
/**
Afegeix un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param p_titol_tema identificador de la pregunta
@param p_descripcio Resposta a la pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirTemaFormacio(
    p_contracte_practiques_id IN contracte_practiques.id%type, 
    p_titol_tema in tema_formacio.titol_tema%type,
    p_descripcio in tema_formacio.descripcio%type,
    RSP out varchar2
) IS
    no_contracte_treball exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_contracte_practiques_id', p_contracte_practiques_id);
    params.put('p_titol_tema', p_titol_tema);
	params.put('p_descripcio', p_descripcio);
	
	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id)  is null) then
		raise no_contracte_treball;
    end if;
	
	insert into tema_formacio (contracte_practiques_id, titol_tema, descripcio  )  
					values(p_contracte_practiques_id, p_titol_tema, p_descripcio ) 
                    returning id into id;
	commit;
	
	-- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
		afegirLog($$plsql_unit, params.to_string, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirTemaFormacio;

/
/**
Afegeix un tema de formacio al contracte del treball
@param p_id 
@param p_titol_tema identificador de la pregunta
@param p_descripcio Resposta a la pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarTemaFormacio(
    p_id in tema_formacio.id%type,
    p_titol_tema in tema_formacio.titol_tema%type,
    p_descripcio in tema_formacio.descripcio%type,
	RSP out varchar2
) IS
    no_tema_formacio exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_titol_tema' value p_titol_tema, 'p_descripcio' value p_descripcio);
	
		-- Cerquem si existeix el contracte de treball
	if (existeixTemaFormacio(p_id) is null) then
		raise no_tema_formacio;
    end if;
	
	update tema_formacio
	set titol_tema = p_titol_tema, 
		descripcio =  p_descripcio
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_tema_formacio then
        RSP := 'ERROR|NO EXISTEIX EL TEMA DE FORMACIO';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarTemaFormacio;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_id 
@param p_titol_tema identificador de la pregunta
@param p_descripcio Resposta a la pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarTemaFormacio(
    p_id in tema_formacio.id%type,
	RSP out varchar2
) IS
    no_tema_formacio exception;
	params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
	
    -- Cerquem si existeix el contracte de treball
	if (existeixTemaFormacio(p_id) is null) then
		raise no_tema_formacio;
    end if;
	
	delete from tema_formacio where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_tema_formacio then
        RSP := 'ERROR|NO EXISTEIX EL TEMA DE FORMACIO';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
		
end eliminarTemaFormacio;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_id 
@param p_valoracio identificador de la pregunta
@param p_data_valoracio Resposta a la pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirTemaFormacioValoracioEstudiant(
    p_id IN tema_formacio.id%type, 
    p_valoracio in tema_formacio.valoracio%type,
    p_data_valoracio in tema_formacio.data_valoracio%type default CURRENT_TIMESTAMP,
	RSP out varchar2
) IS
    no_tema_formacio exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_valoracio' value p_valoracio, 'p_data_valoracio' value p_data_valoracio );
	
		-- Cerquem si existeix el contracte de treball
	if (existeixTemaFormacio(p_id) is null) then
		raise no_tema_formacio;
    end if;
	
	update tema_formacio 
	set valoracio = p_valoracio, data_valoracio = p_data_valoracio
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_tema_formacio then
        RSP := 'ERROR|NO EXISTEIX EL TEMA DE FORMACIO';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end afegirTemaFormacioValoracioEstudiant;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param p_comentari_tutor identificador de la pregunta
@param p_data_informe Resposta a la pregunta feta per la companyia a l'estudiant
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirInformeParcial(
    p_contracte_practiques_id IN contracte_practiques.id%type,     
    p_comentari_tutor in informe_parcial.comentari_tutor%type,
    p_data_informe in informe_parcial.data_informe%type default current_timestamp,
    RSP out varchar2
) IS
    no_contracte_treball exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_contracte_practiques_id', p_contracte_practiques_id);
    params.put('p_comentari_tutor', p_comentari_tutor);
	params.put('p_data_informe', p_data_informe);
	
	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id) is null) then
		raise no_contracte_treball;
    end if;
    
	insert into informe_parcial (contracte_practiques_id, comentari_tutor, data_informe  )  
					values(p_contracte_practiques_id, p_comentari_tutor, p_data_informe ) 
                    returning id into id;
	commit;

    -- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
	
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
		afegirLog($$plsql_unit, params.to_string, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirInformeParcial;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_id 
@param p_comentari_tutor identificador de la pregunta
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarInformeParcial(
    p_id in informe_parcial.id%type,
    p_comentari_tutor in informe_parcial.comentari_tutor%type,
	RSP out varchar2
) IS
    no_informe_parcial exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_comentari_tutor' value p_comentari_tutor);
	
	-- Cerquem si existeix un informe parcial
	if (existeixInformeParcial(p_id) is null) then
		raise no_informe_parcial;
    end if;
	
	update informe_parcial 
	set comentari_tutor = p_comentari_tutor
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_informe_parcial then
        RSP := 'ERROR|NO EXISTEIX L''INFORME PARCIAL';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarInformeParcial;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_id 
@param p_comentari_tutor identificador de la pregunta
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarInformeParcial(
    p_id in informe_parcial.id%type,
	RSP out varchar2
) IS
    no_informe_parcial exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
	
	-- Cerquem si existeix un informe parcial
	if (existeixInformeParcial(p_id) is null) then
		raise no_informe_parcial;
    end if;
	
	delete from informe_parcial where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_informe_parcial then
        RSP := 'ERROR|NO EXISTEIX L''INFORME PARCIAL';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarInformeParcial;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param p_comentari_tutor identificador de la pregunta
@param p_es_apte identificador de la pregunta
@param p_data_informe identificador de la pregunta
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirInformeFinal(
    p_contracte_practiques_id IN contracte_practiques.id%type, 
    p_comentari_tutor in contracte_practiques.comentari_tutor%type,
    p_es_apte in contracte_practiques.es_apte%type,
    p_data_informe in contracte_practiques.data_informe%type default CURRENT_TIMESTAMP,
	RSP out varchar2
) IS
    no_contracte_treball exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_contracte_practiques_id' value p_contracte_practiques_id, 'p_comentari_tutor' value p_comentari_tutor, 'p_es_apte' value p_es_apte, 'p_data_informe' value p_data_informe);
	
	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id) is null) then
		raise no_contracte_treball;
    end if;
	
	
	update contracte_practiques
	set comentari_tutor = p_comentari_tutor, 
		data_informe = p_data_informe, 
		es_apte = p_es_apte
	where id = p_contracte_practiques_id;
   
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
    
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
	
end afegirInformeFinal;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarInformeFinal(
    p_contracte_practiques_id IN contracte_practiques.id%type,
	RSP out varchar2
) IS
    no_contracte_treball exception;
    p_es_apte number;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_contracte_practiques_id' value p_contracte_practiques_id);
	
	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id) is null) then
		raise no_contracte_treball;
    end if;
    
	update contracte_practiques 
	set comentari_tutor = null, 
		data_informe = null, 
		es_apte = null
	where id = p_contracte_practiques_id;
	commit;

	afegirLog($$plsql_unit, params, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end eliminarInformeFinal;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirInformeFinalComentariAlumne(
    p_contracte_practiques_id IN contracte_practiques.id%type, 
    p_comentari_alumne in contracte_practiques.comentari_alumne%type,
	RSP out varchar2
) IS
    no_contracte_treball exception;
	params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_contracte_practiques_id' value p_contracte_practiques_id, 'p_comentari_alumne' value p_comentari_alumne);
	
	-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id) is null) then
		raise no_contracte_treball;
    end if;
	
	update contracte_practiques
	set comentari_alumne = p_comentari_alumne
	where id = p_contracte_practiques_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);	
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);	
end afegirInformeFinalComentariAlumne;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirInspeccioTreball(
    p_contracte_practiques_id IN contracte_practiques.id%type, 
    p_es_favorable in inspeccio_treball.es_favorable%type,
    p_comentari in inspeccio_treball.comentari%type,
    p_data_inspeccio in inspeccio_treball.data_inspeccio%type default CURRENT_TIMESTAMP,
    RSP out varchar2
) IS
    no_contracte_treball exception;
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_contracte_practiques_id', p_contracte_practiques_id);
    params.put('p_es_favorable', p_es_favorable);
	params.put('p_comentari', p_comentari);
    params.put('p_data_inspeccio', p_data_inspeccio);
	
		-- Cerquem si existeix el contracte de treball
	if (existeixContracteTreball(p_contracte_practiques_id) is null) then
		raise no_contracte_treball;
    end if;
	
	insert into inspeccio_treball (contracte_practiques_id, es_favorable, comentari, data_inspeccio  )  
					values(p_contracte_practiques_id, p_es_favorable, p_comentari, p_data_inspeccio ) 
                    returning id into id;
	commit;
	
   -- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
	when no_contracte_treball then
        RSP := 'ERROR|NO EXISTEIX EL CONTRACTE DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);	
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);
end afegirInspeccioTreball;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarInspeccioTreball(
    p_id in inspeccio_treball.id%type,
    p_es_favorable in inspeccio_treball.es_favorable%type,
    p_comentari in inspeccio_treball.comentari%type,
    p_data_inspeccio in inspeccio_treball.data_inspeccio%type default CURRENT_TIMESTAMP,
	RSP out varchar2
) IS
    no_valors_booleans exception;
    no_inspeccio_treball exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_es_favorable' value p_es_favorable, 'p_comentari' value p_comentari, 'p_data_inspeccio' value p_data_inspeccio);
	
    -- Comprovem valors esFavorable
	if (comprovaValorsBooleans(p_es_favorable) = false) then
		raise no_valors_booleans;
    end if;
    
	-- Cerquem si existeix un informe parcial
	if (existeixInspeccioTreball(p_id) is null) then
		raise no_inspeccio_treball;
    end if;
	
	update inspeccio_treball
	set es_favorable = p_es_favorable, 
		comentari = p_comentari, 
		data_inspeccio = p_data_inspeccio
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_valors_booleans then
        RSP := 'ERROR|EL VALOR PER ES_FAVORABLE NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params, RSP);
        
    when no_inspeccio_treball then
        RSP := 'ERROR|NO EXISTEIX LA INSPECCIÓ DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarInspeccioTreball;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarInspeccioTreball(
    p_id in inspeccio_treball.id%type,
	RSP out varchar2
) IS
    no_inspeccio_treball exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
	
	-- Cerquem si existeix un informe parcial
	if (existeixInspeccioTreball(p_id) is null) then
		raise no_inspeccio_treball;
    end if;
	
	delete from inspeccio_treball where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_inspeccio_treball then
        RSP := 'ERROR|NO EXISTEIX LA INSPECCIÓ DE TREBALL';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarInspeccioTreball;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure afegirInspeccioTreballMostreig(
    p_inspeccio_treball_id IN inspeccio_treball.id%type, 
    p_titol_mostreig in inspeccio_treball_mostreig.titol_mostreig%type,
    p_es_correcte in inspeccio_treball_mostreig.es_correcte%type,
    RSP out varchar2
) IS
    no_valors_booleans exception;
    no_inspeccio_treball exception;
    
    id number;
    params json_object_t;
begin
    -- Inicialitzem la variable RSP
    RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT_T( '{"id":null}');
    params.put('p_inspeccio_treball_id', p_inspeccio_treball_id);
    params.put('p_titol_mostreig', p_titol_mostreig);
	params.put('p_es_correcte', p_es_correcte);
    
     -- Comprovem valors esFavorable
	if (comprovaValorsBooleans(p_es_correcte) = false) then
		raise no_valors_booleans;
    end if;
	
	-- Cerquem si existeix una inspecci de treball
	if (existeixInspeccioTreball(p_inspeccio_treball_id) is null) then
		raise no_inspeccio_treball;
    end if;
	
	insert into inspeccio_treball_mostreig (inspeccio_treball_id, titol_mostreig, es_correcte  )  
								 values(p_inspeccio_treball_id, p_titol_mostreig, p_es_correcte ) 
								returning id into id;
	commit;

    -- afegim l'identificador al JSON
    params.put('id', id);
    afegirLog($$plsql_unit, params.to_string, RSP);
exception
    when no_valors_booleans then
        RSP := 'ERROR|EL VALOR PER ES_CORRECTE NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params.to_string, RSP);
	when no_inspeccio_treball then
        RSP := 'ERROR|NO EXISTEIX LA INSPECCIÓ DE TREBALL';
        afegirLog($$plsql_unit, params.to_string, RSP);
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params.to_string, RSP);	
end afegirInspeccioTreballMostreig;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_contracte_practiques_id 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure modificarInspeccioTreballMostreig(
    p_id in inspeccio_treball_mostreig.id%type,
    p_titol_mostreig in inspeccio_treball_mostreig.titol_mostreig%type,
    p_es_correcte in inspeccio_treball_mostreig.es_correcte%type,
	RSP out varchar2
) IS
    no_valors_booleans exception;
    no_mostreig_inspeccio_treball exception;
	params varchar2(2000);
begin

	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id, 'p_titol_mostreig' value p_titol_mostreig, 'p_es_correcte' value p_es_correcte);
	
     -- Comprovem valors esFavorable
	if (comprovaValorsBooleans(p_es_correcte) = false) then
		raise no_valors_booleans;
    end if;
    
	-- Cerquem si existeix mostreig de la inspecci de treball
	if (existeixMostreigInspeccioTreball(p_id) is null) then
		raise no_mostreig_inspeccio_treball;
    end if;
	
	update inspeccio_treball_mostreig
	set titol_mostreig = p_titol_mostreig, 
		es_correcte = p_es_correcte
	where id = p_id;
	commit;
	
	afegirLog($$plsql_unit, params, RSP);
exception
    when no_valors_booleans then
        RSP := 'ERROR|EL VALOR PER ES_CORRECTE NO ES VLID [0,1]';
        afegirLog($$plsql_unit, params, RSP);
    when no_mostreig_inspeccio_treball then
        RSP := 'ERROR|NO EXISTEIX EL MOSTREIG';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end modificarInspeccioTreballMostreig;

/
/**
Elimina un tema de formacio al contracte del treball
@param p_id identificador del mostreig 
@param RSP OUT retorna el resultat de l'execuci del procediment
**/
create or replace procedure eliminarInspeccioTreballMostreig(
    p_id in inspeccio_treball_mostreig.id%type,
	RSP out varchar2
) IS
    no_mostreig_inspeccio_treball exception;
    
	params varchar2(2000);
begin
	-- Inicialitzem la variable RSP
	RSP := 'OK';
    -- Creem el JSON per al registre
    params := JSON_OBJECT('p_id' value p_id);
	
	-- Cerquem si existeix mostreig de la inspecci de treball
	if (existeixMostreigInspeccioTreball(p_id) is null) then
		raise no_mostreig_inspeccio_treball;
    end if;
	
	delete from inspeccio_treball_mostreig where id = p_id;
	commit;
	afegirLog($$plsql_unit, params, RSP);
	
exception
    when no_mostreig_inspeccio_treball then
        RSP := 'ERROR|NO EXISTEIX EL MOSTREIG';
        afegirLog($$plsql_unit, params, RSP);
        
    when others then
        RSP := concat('ERROR|', SQLERRM);
        afegirLog($$plsql_unit, params, RSP);
end eliminarInspeccioTreballMostreig;

/