/** Eliminem les taules i les dades **/
drop table e_estudiants_dades;
drop table e_estudiants_per_universitat;
drop table e_estudiants_any_universitari;
drop table e_companyia_dades_i_any_universitari;
drop table e_companyia_dades;
drop table e_titulacio_dades;
drop table e_professor_i_any_universitari;
drop table e_estudiants_per_universitat_i_any_universitari;
drop table e_any_universitari;

drop table informe_parcial purge;
drop table tema_formacio purge;
drop table inspeccio_treball_mostreig purge;
drop table inspeccio_treball purge;
drop table contracte_practiques purge;
drop table pregunta_entrevista purge;
drop table candidat purge;
drop table oferta_treball purge;
drop table universitat_titulacio purge;
drop table estudiant purge;
drop table professor purge;
drop table treballador purge;
drop table departament purge;
drop table companyia purge;
drop table titulacio purge;
drop table universitat purge;


drop table log_accions purge;
drop table log_accions_llista purge;

/** Creem les taules del log **/
create table log_accions_llista(
    id number GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    nom_accio varchar2(50) not null,
    CONSTRAINT logAccionsLlistaPK PRIMARY KEY (id),
    CONSTRAINT logAccionsLlistaUNIQUE UNIQUE (nom_accio)
);

create table log_accions(
    id NUMBER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    data_peticio timestamp not null,
    log_accio_llista_id number not null,
    sortida varchar2(1000) not null,
    parametre_entrada varchar2(2000),
    CONSTRAINT logAccionsPK PRIMARY KEY (id),
    CONSTRAINT logAccionsLlistatFK FOREIGN KEY (log_accio_llista_id) REFERENCES log_accions_llista (id) ON DELETE CASCADE ENABLE
);

/*Creem les taules de les entitats principals*/
CREATE TABLE companyia(
    cif CHAR(9) not null,
    nom VARCHAR2(50) not null,
    data_peticio_inscripcio timestamp not null,
    data_revisio_inscripcio timestamp default null,
    es_acceptada number(1) default null,
    CONSTRAINT companyiaPK PRIMARY KEY (cif),
    CONSTRAINT companyiaEsAcceptada CHECK (es_acceptada = 0 or es_acceptada = 1) ENABLE
);

create table departament(
    id SMALLINT GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    nom VARCHAR2(50) not null,
    CONSTRAINT departamentPK PRIMARY KEY (id)
);

create table titulacio(
    id SMALLINT GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    nom VARCHAR2(50) not null,
    CONSTRAINT titulacioPK PRIMARY KEY (id)
);

create table universitat(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    nom VARCHAR2(50) not null,
    te_estudiant_per_practiques NUMBER(1) DEFAULT 0 NOT NULL,
    CONSTRAINT universitatPK PRIMARY KEY (id),
     CONSTRAINT universitatTeEstudiantPerPractiquesCheck CHECK (te_estudiant_per_practiques =0 or te_estudiant_per_practiques=1) ENABLE
);


/*Creem les taules de les entitats secundaries*/

create table treballador(
    nif CHAR(9) not null,
    companyia_cif char(9) not null,
    departament_id SMALLINT not null,
    permis SMALLINT default 0  not null,
    nom VARCHAR2(50) not null,
    cognoms VARCHAR2(255) not null,
    CONSTRAINT treballadorPK PRIMARY KEY (nif),
    CONSTRAINT treballadorCompanyiaFK FOREIGN KEY (companyia_cif) REFERENCES companyia (cif) ON DELETE CASCADE ENABLE,
    CONSTRAINT treballadorDepartamentFK FOREIGN KEY (departament_id) REFERENCES departament (id) ON DELETE CASCADE ENABLE
);

create table professor(
    nif char(9) not null,
    universitat_id INTEGER not null,
    nom varchar(50) not null,
    cognoms varchar2(255) not null,
    CONSTRAINT professorPK PRIMARY KEY (nif),
    CONSTRAINT professorUniversitatFK FOREIGN KEY (universitat_id) REFERENCES universitat (id) ON DELETE CASCADE ENABLE
);

CREATE table estudiant(
    nif char(9) not null,
    universitat_id INTEGER not null,
    titulacio_id SMALLINT not null,
    nom varchar2(50) not null,
    cognoms varchar2(255) not null,
    CONSTRAINT estudiantPK PRIMARY KEY (nif),
    CONSTRAINT estudiantUniversitatFK FOREIGN KEY (universitat_id) REFERENCES universitat (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT estudiantTitulacioFK FOREIGN KEY (titulacio_id) REFERENCES titulacio (id) ON DELETE CASCADE ENABLE

);

create table universitat_titulacio(
    universitat_id INTEGER NOT NULL,
    titulacio_id SMALLINT not null,
    CONSTRAINT universitatTitulacioUniversitatFK FOREIGN KEY (universitat_id) REFERENCES universitat (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT universitatTitulacioTitulacioFK FOREIGN KEY (titulacio_id) REFERENCES titulacio (id) ON DELETE CASCADE ENABLE
);


/*Creem les taules de les relacions entre entitats*/

CREATE TABLE oferta_treball(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    companyia_cif CHAR(9) not null,
    universitat_id INTEGER not null,
    titulacio_id SMALLINT not null,
    data_creacio timestamp not null,
    process_finalitzat number(1) default 0 not null ,
    nom varchar2(50) not null,
    descripcio_requisits VARCHAR2(2000) not null,
    CONSTRAINT ofertaTreballPK PRIMARY KEY (id),
    CONSTRAINT ofertaTreballCompanyiaFK FOREIGN KEY (companyia_cif) REFERENCES companyia (cif) ON DELETE CASCADE ENABLE,
    CONSTRAINT ofertaTreballUniversitatFK FOREIGN KEY (universitat_id) REFERENCES universitat (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT ofertaTreballTitulacioFK FOREIGN KEY (titulacio_id) REFERENCES titulacio (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT ofertaTreballProcessFinalitzatCheck CHECK (process_finalitzat =0 or process_finalitzat=1) ENABLE
);

create table candidat(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    estudiant_nif char(9) not null,
    oferta_treball_id INTEGER not null,
    data_inscripcio timestamp not null,
    validat_per char(9),
    seleccionat_per char(9),
    data_enviada_empresa TIMESTAMP,
    data_seleccionat timestamp,
    es_apte NUMBER(1),
    data_validat timestamp,
    requisits_empresa varchar(2000),
    CONSTRAINT candidatPK PRIMARY KEY (id),
    CONSTRAINT candidatEstudiantFK FOREIGN KEY (estudiant_nif) REFERENCES estudiant (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT candidatOfertaTreballFK FOREIGN KEY (oferta_treball_id) REFERENCES oferta_treball (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT candidatTreballadorValidatFK FOREIGN KEY (validat_per) REFERENCES treballador (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT candidatTreballadorSeleccionatFK FOREIGN KEY (seleccionat_per) REFERENCES treballador (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT candidatEsApteCheck CHECK (es_apte =0 or es_apte =1) ENABLE
);

create table pregunta_entrevista(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    candidat_id INTEGER not null,
    data_pregunta timestamp not null,
    data_resposta timestamp,
    pregunta varchar2(200) not null,
    resposta varchar2(400),
    CONSTRAINT preguntaEntrevistaPK PRIMARY KEY (id),
    CONSTRAINT preguntaEntrevistaCandidaturaFK FOREIGN KEY (candidat_id) REFERENCES candidat (id) ON DELETE CASCADE ENABLE
);

create table contracte_practiques(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    oferta_treball_id INTEGER not null,
    estudiant_nif char(9) not null,
    professor_nif char(9) not null,
    treballador_nif char(9) not null,
    data_inici timestamp not null,
    data_fi timestamp not null,
    sou number(6,2) not null,
    comentari_tutor varchar2(2000),
    comentari_alumne varchar2(2000) ,
    data_informe timestamp,
    es_apte number(1) default 0,
    CONSTRAINT contractePractiquesPK PRIMARY KEY (id),
    CONSTRAINT contractePractiquesOfertaTreballFK FOREIGN KEY (oferta_treball_id) REFERENCES oferta_treball (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT contractePractiquesEstudiantFK FOREIGN KEY (estudiant_nif) REFERENCES estudiant (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT contractePractiquesProfessorFK FOREIGN KEY (professor_nif) REFERENCES professor (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT contractePractiquesTreballadorFK FOREIGN KEY (treballador_nif) REFERENCES treballador (nif) ON DELETE CASCADE ENABLE,
    CONSTRAINT contractePractiquesEsApte CHECK (es_apte = 0 or es_apte = 1) ENABLE
);

create table inspeccio_treball(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    contracte_practiques_id INTEGER not null,
    data_inspeccio timestamp not null,
    es_favorable number(1) not null,
    comentari varchar2(2000) not null,
    CONSTRAINT inspeccioTreballPK PRIMARY KEY (id),
    CONSTRAINT inspeccioTreballFK FOREIGN KEY (contracte_practiques_id) REFERENCES contracte_practiques (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT inspeccioTreballEsFavorableCheck CHECK (es_favorable =0 or es_favorable=1) ENABLE
);

create table inspeccio_treball_mostreig(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    inspeccio_treball_id INTEGER not null,
    titol_mostreig varchar(2000) not null,
    es_correcte number(1) default 0 not null ,
    CONSTRAINT inspeccioTreballMostreigPK PRIMARY KEY (id),
    CONSTRAINT inspeccioTreballMostreigInspeccioTreballFK FOREIGN KEY (inspeccio_treball_id) REFERENCES inspeccio_treball (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT inspeccioTreballMostreigEsCorrecteCheck CHECK (es_correcte =0 or es_correcte=1) ENABLE
);

create table tema_formacio(
    id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    contracte_practiques_id INTEGER not null,
    titol_tema varchar2(250) not null,
    descripcio varchar2(2000) not null,
    valoracio smallint ,
    data_valoracio timestamp ,
    CONSTRAINT temaFormacioPK PRIMARY KEY (id),
    CONSTRAINT temaFormacioContractePractiquesFK FOREIGN KEY (contracte_practiques_id) REFERENCES contracte_practiques (id) ON DELETE CASCADE ENABLE,
    CONSTRAINT temaFormacioValoracioCheck CHECK (valoracio >= 0 and  valoracio < 6) ENABLE
);

create table informe_parcial(
     id INTEGER GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
     contracte_practiques_id INTEGER not null,
     data_informe timestamp not null,
     comentari_tutor varchar2(2000) not null,
     CONSTRAINT informeParcialPK PRIMARY KEY (id),
     CONSTRAINT informeParcialContractePractiquesFK FOREIGN KEY (contracte_practiques_id) REFERENCES contracte_practiques (id) ON DELETE CASCADE ENABLE
);

/**** taules d'estadstiques ***/
create table e_any_universitari(
    id number GENERATED ALWAYS AS IDENTITY MINVALUE 1 INCREMENT BY 1 START WITH 1 NOT NULL,
    any_universitari char(5) not null,
    CONSTRAINT E_AnyUniversitariPK PRIMARY KEY (id),
    CONSTRAINT E_AnyUniversitariUNIQUE UNIQUE (any_universitari)
);

create table e_estudiants_dades(
    id number(1) not null,
    total_estudiants integer default 0 not null,
    total_ofertes_treball integer default 0 not null,
    estudiants_presentats integer default 0 not null,
    estudiants_validats integer default 0 not null,
    estudiants_seleccionats integer default 0 not null,
    estudiants_finalitzats integer default 0 not null,
    estudiants_aprovats integer default 0 not null,
    total_contractes integer default 0 not null,
    dies_total_contracte number default 0 not null,
    sou_total number default 0 not null,
    CONSTRAINT E_EstudiantDadesPK PRIMARY KEY (id)
);

create table e_estudiants_per_universitat(
    universitat_id integer,
    total_estudiants integer default 0 not null,
    total_ofertes_treball integer default 0 not null,
    estudiants_presentats integer default 0 not null,
    estudiants_validats integer default 0 not null,
    estudiants_seleccionats integer default 0 not null,
    estudiants_finalitzats integer default 0 not null,
    estudiants_aprovats integer default 0 not null,
    total_contractes integer default 0 not null,
    dies_total_contracte number default 0 not null,
    sou_total number default 0 not null,
    CONSTRAINT E_EstudiantsPerUniversitatPK PRIMARY KEY (universitat_id)
);

create table e_estudiants_per_universitat_i_any_universitari(
    any_universitari_id number not null,
    universitat_id integer not null,
    total_ofertes_treball integer default 0 not null,
    estudiants_presentats integer default 0 not null,
    estudiants_validats integer default 0 not null,
    estudiants_seleccionats integer default 0 not null,
    estudiants_finalitzats integer default 0 not null,
    estudiants_aprovats integer default 0 not null,
    total_contractes integer default 0 not null,
    dies_total_contracte number default 0 not null,
    sou_total number default 0 not null,
    CONSTRAINT E_EstudiantsPerUniversitatIAnyUniversitariPK PRIMARY KEY (universitat_id, any_universitari_id),
    CONSTRAINT E_EstudiantsPerUniversitatIAnyUniversitariAnyUniversitariFK FOREIGN KEY (any_universitari_id) REFERENCES e_any_universitari (id) ON DELETE CASCADE ENABLE
);

create table e_estudiants_any_universitari(
    any_universitari_id number not null,
    total_ofertes_treball integer default 0 not null,
    estudiants_presentats integer default 0 not null,
    estudiants_validats integer default 0 not null,
    estudiants_seleccionats integer default 0 not null,
    estudiants_finalitzats integer default 0 not null,
    estudiants_aprovats integer default 0 not null,
    total_contractes integer default 0 not null,
    dies_total_contracte number default 0 not null,
    sou_total number default 0 not null,
    CONSTRAINT E_EstudiantsAnyUniversitariPK PRIMARY KEY (any_universitari_id),
    CONSTRAINT E_EstudiantsAnyUniversitariAnyUniversitariFK FOREIGN KEY (any_universitari_id) REFERENCES e_any_universitari (id) ON DELETE CASCADE ENABLE
);

create table e_companyia_dades(
    companyia_cif char(9) not null ,
    total_ofertes_treball number default 0 not null,
    total_entrevistes number default 0 not null,
    total_inspeccions_rebudes number default 0 not null,
    CONSTRAINT E_CompanyiaDadesPK PRIMARY KEY (companyia_cif)
);

create table e_companyia_dades_i_any_universitari(
    companyia_cif char(9) not null ,
    any_universitari_id number not null,
    total_ofertes_treball number default 0 not null,
    total_inspeccions_rebudes number default 0 not null,
    CONSTRAINT E_CompanyiaDadesIAnyUniversitariPK PRIMARY KEY (companyia_cif, any_universitari_id),
    CONSTRAINT E_CompanyiaDadesIAnyUniversitariAnyUniversitariFK FOREIGN KEY (any_universitari_id) REFERENCES e_any_universitari (id) ON DELETE CASCADE ENABLE
);

create table e_professor_i_any_universitari(
    any_universitari_id number not null,
    professor_nif char(9) not null,
    total_estudiants_en_practiques number default 0 not null,
    total_estudiants_aprovats number default 0 not null,
    CONSTRAINT E_EstudiantsPractiquesAnyUniversitariPK PRIMARY KEY (any_universitari_id, professor_nif),
    CONSTRAINT E_EstudiantsPractiquesAnyUniversitariAnyUniversitariFK FOREIGN KEY (any_universitari_id) REFERENCES e_any_universitari (id) ON DELETE CASCADE ENABLE
);

create table e_titulacio_dades(
    titulacio_id number not null ,
    total_estudiants_en_practiques_seleccionats number default 0 not null,
    total_estudiants_en_practiques_finalitzat_satisfactori number default 0 not null,
    CONSTRAINT E_TitulacioDadesPK PRIMARY KEY (titulacio_id)
);

/** Index **/
CREATE INDEX candidat_index1 ON candidat (oferta_treball_id, es_apte) ;
CREATE INDEX candidat_index2 ON candidat (id, oferta_treball_id);
CREATE INDEX candidat_index3 ON candidat (estudiant_nif, es_apte);
CREATE INDEX oferta_treball_index1 ON oferta_treball (id, universitat_id); 
CREATE INDEX oferta_treball_index2 ON oferta_treball (id, companyia_cif); 
CREATE INDEX oferta_treball_index3 ON oferta_treball (id, process_finalitzat) ;
CREATE INDEX companyia_index1 ON companyia (cif, es_acceptada) ;
CREATE INDEX estudiant_index1 ON estudiant (nif, titulacio_id) ;

CREATE INDEX e_estudiants_per_universitat_index1 ON e_estudiants_per_universitat (universitat_id, estudiants_finalitzats); 

 

commit;