/**
Disparador que s'executa quan s'actualitza una 
de les segents columnes a la taula candidat:
 - data_enviada_empresa
 - data_validat
 - es_apte
**/
create or replace NONEDITIONABLE trigger e_candidat
    after update of data_enviada_empresa, data_validat, es_apte  on candidat

    for each row
declare
    any_universitari number;
    p_universitat_id number;
    p_companyia_cif char(9);
    p_data timestamp;
begin
    select universitat_id, data_creacio, companyia_cif 
            into p_universitat_id, p_data, p_companyia_cif
    from oferta_treball
    where id = :NEW.oferta_treball_id;

    any_universitari :=  buscaAnyUniversitari( p_data => p_data );


    case 
        /* actualitzem els estudiants presentats quan la universitat envia la 
            candidatura a la companyia */ 
        when updating('data_enviada_empresa') then

             if(:NEW.data_enviada_empresa is not null and :OLD.data_enviada_empresa is null) then
                /** total de candidats per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_presentats = estudiants_presentats + 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total de candidats per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_presentats = estudiants_presentats + 1
                where any_universitari_id = any_universitari;
                
                /** total de candidats per any universitari **/
                update e_estudiants_dades
                set estudiants_presentats = estudiants_presentats + 1
                where id = 1;

                /** total de candidats per universitat **/
                update e_estudiants_per_universitat
                set estudiants_presentats = estudiants_presentats + 1
                where universitat_id = p_universitat_id;

            elsif (:NEW.data_enviada_empresa is null and :OLD.data_enviada_empresa is not null) then

                /** total de candidats per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_presentats = estudiants_presentats - 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;

                /** total de candidats per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_presentats = estudiants_presentats - 1
                where any_universitari_id = any_universitari;

                /** total de candidats per any universitari **/
                update e_estudiants_dades
                set estudiants_presentats = estudiants_presentats - 1
                where id = 1;
                
                /** total de candidats per universitat **/
                update e_estudiants_per_universitat
                set estudiants_presentats = estudiants_presentats - 1
                where universitat_id = p_universitat_id;
            end if;


         /* actualitzem els estudiants validats per la companyia*/ 
        when updating('data_validat') then

            if(:NEW.data_validat is not null and :OLD.data_validat is null) then

                /** total de candidats validats per empresa per universitat i any universitari **/            
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_validats = estudiants_validats + 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;

                /** total de candidats validats per empresa per any universitari **/            
                update e_estudiants_any_universitari
                set estudiants_validats = estudiants_validats + 1
                where any_universitari_id = any_universitari;
                
                /** total de candidats validats per empresa per any universitari **/            
                update e_estudiants_dades
                set estudiants_validats = estudiants_validats + 1
                where id = 1;
                
                /** total de candidats validats per empresa per universitat **/            
                update e_estudiants_per_universitat
                set estudiants_validats = estudiants_validats + 1
                where universitat_id = p_universitat_id;

            elsif (:NEW.data_validat is null and :OLD.data_validat is not null) then
                /** total de candidats validats per empresa per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_validats = estudiants_validats - 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total de candidats validats per empresa per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_validats = estudiants_validats - 1
                where any_universitari_id = any_universitari;
                
                /** total de candidats validats per empresa per any universitari **/
                update e_estudiants_dades
                set estudiants_validats = estudiants_validats - 1
                where id = 1;

                /** total de candidats validats per empresa per universitat **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_validats = estudiants_validats - 1
                where universitat_id = p_universitat_id;

            end if;

         /* actualitzem els estudiants seleccionats per la companyia*/ 
        when updating('es_apte') then

           update e_companyia_dades
           set total_entrevistes = total_entrevistes + 1
           where companyia_cif = p_companyia_cif;

            if(:NEW.es_apte = 1 and ( :OLD.es_apte = 0 or :OLD.es_apte is null) ) then
                 /** total de candidats seleccionats per empresa per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_seleccionats = estudiants_seleccionats + 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;

                /** total de candidats seleccionats per empresa per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_seleccionats = estudiants_seleccionats + 1
                where any_universitari_id = any_universitari;
                
                /** total de candidats seleccionats per empresa per any universitari **/
                update e_estudiants_dades
                set estudiants_seleccionats = estudiants_seleccionats + 1
                where id = 1;
                
                 /** total de candidats seleccionats per empresa per universitat **/
                update e_estudiants_per_universitat
                set estudiants_seleccionats = estudiants_seleccionats + 1
                where universitat_id = p_universitat_id;

            elsif (:OLD.es_apte = 1 and :NEW.es_apte = 0) then
                /** total de candidats seleccionats per empresa per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_seleccionats = estudiants_seleccionats - 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;

                /** total de candidats seleccionats per empresa per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_seleccionats = estudiants_seleccionats - 1
                where any_universitari_id = any_universitari;
                
                /** total de candidats seleccionats per empresa per any universitari **/
                update e_estudiants_dades
                set estudiants_seleccionats = estudiants_seleccionats - 1
                where id = 1;

                /** total de candidats seleccionats per empresa per universitat i any universitari **/
                update e_estudiants_per_universitat
                set estudiants_seleccionats = estudiants_seleccionats - 1
                where universitat_id = p_universitat_id;

            end if;


        end case;

end;



/
/**
Disparador que s'executa quan s'eliminen  
dades a la taula candidat
**/

create or replace NONEDITIONABLE trigger e_candidat_delete
    before  delete  on candidat

    for each row
declare
    /** es necessari per poder capturar quan s'elimina una dada que, mitjanant
    l'integritat referencial de la dada, fa que s'elimini la dada de candidat. 
    Per exemple, quan s'elimina una oferta de treball **/
    pragma autonomous_transaction;

    any_universitari number;
    p_universitat_id number;
    p_data timestamp;
    p_companyia_cif char(9);

    RSP varchar2(2000);
begin

    select universitat_id, data_creacio, companyia_cif
            into p_universitat_id, p_data, p_companyia_cif
    from oferta_treball
    where id = :OLD.oferta_treball_id;

    any_universitari :=  buscaAnyUniversitari( p_data => p_data );


    if(:OLD.data_enviada_empresa is not null ) then
        /** total de candidats per universitat i any universitari **/
        begin
            merge into e_estudiants_per_universitat_i_any_universitari desti
            using dual
                on ( desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_presentats = estudiants_presentats - 1
                      where desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

        /** total de candidats per any universitari **/
        begin
            merge into e_estudiants_any_universitari desti
            using dual
                on ( any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_presentats = estudiants_presentats - 1
                      where any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total de candidats per any universitari **/
        begin
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1 )
            when matched then update set desti.estudiants_presentats = estudiants_presentats - 1
                      where id = 1;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

        /** total de candidats per universitat **/
        begin
            merge into e_estudiants_per_universitat desti
            using dual
                on ( desti.universitat_id = p_universitat_id )
            when matched then update set desti.estudiants_presentats = estudiants_presentats + 1
                      where desti.universitat_id = p_universitat_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

    end if;

    if(:OLD.data_validat is not null ) then
         /** total de candidats validats per empresa per universitat i any universitari**/
         begin
            merge into e_estudiants_per_universitat_i_any_universitari desti
            using dual
                on ( desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_validats = estudiants_validats - 1
                      where desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

         /** total de candidats validats per empresa per any universitari**/
         begin
            merge into e_estudiants_any_universitari desti
            using dual
                on ( any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_validats = estudiants_validats - 1
                      where  any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total de candidats validats per empresa per any universitari**/
         begin
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1 )
            when matched then update set desti.estudiants_validats = estudiants_validats - 1
                      where  id = 1;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        

         /** total de candidats validats per empresa per universitat **/
        begin
            merge into e_estudiants_per_universitat desti
            using dual
                on ( desti.universitat_id = p_universitat_id )
            when matched then update set desti.estudiants_validats = estudiants_validats - 1
                      where desti.universitat_id = p_universitat_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

    end if;
    
    if(:OLD.es_apte = 1 or :OLD.es_apte = 0)then
        update e_companyia_dades
        set total_entrevistes = total_entrevistes - 1
        where companyia_cif = p_companyia_cif;
    end if;

    if(:OLD.es_apte = 1 ) then
         /** total de candidats seleccionats per empresa per universitat i any universitari **/
         begin
            merge into e_estudiants_per_universitat_i_any_universitari desti
            using dual
                on ( desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_seleccionats = estudiants_seleccionats - 1
                      where desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

        /** total de candidats seleccionats per empresa per any universitari **/
         begin
            merge into e_estudiants_any_universitari desti
            using dual
                on ( any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_seleccionats = estudiants_seleccionats - 1
                      where any_universitari_id = any_universitari;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total de candidats seleccionats per empresa per any universitari **/
        begin
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1 )
            when matched then update set desti.estudiants_seleccionats = estudiants_seleccionats - 1
                      where id = 1;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        

        /** total de candidats seleccionats per empresa per universitat**/
        begin
            merge into e_estudiants_per_universitat desti
            using dual
                on ( desti.universitat_id = p_universitat_id )
            when matched then update set desti.estudiants_seleccionats = estudiants_seleccionats - 1
                      where desti.universitat_id = p_universitat_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

    end if;

    commit;

end;



/
/**
Disparador que s'executa quan s'eliminen 
dades a la taula companyia:
**/
CREATE OR REPLACE TRIGGER E_COMPANYIA_DELETE 
after DELETE ON companyia
for each row
BEGIN
    delete from e_companyia_dades_i_any_universitari where companyia_cif = :OLD.cif ;
    delete from e_companyia_dades where companyia_cif = :OLD.cif ;
END;

/
/**
Disparador que s'executa quan s'afegeixen dades o
s'actualitza una  de les segents columnes a la taula contracte_practiques:
 - data_informe
 - es_apte
**/
create or replace NONEDITIONABLE trigger e_contracte_practiques
    after insert or
        update of es_apte, data_informe on contracte_practiques

    for each row
declare
    any_universitari number;
    p_universitat_id number;
    p_titulacio_id number;
    p_data timestamp;
begin

    select universitat_id, titulacio_id, data_creacio
        into p_universitat_id, p_titulacio_id, p_data
    from oferta_treball
    where id = :NEW.oferta_treball_id;

    any_universitari :=  buscaAnyUniversitari( p_data => p_data );

    case 
        when inserting then
        
            /** totals d'estudiants en practiques per universitat **/
            merge into e_professor_i_any_universitari desti
            using dual
                on ( desti.any_universitari_id = any_universitari  and professor_nif = :NEW.professor_nif)
            when matched then update 
                            set desti.total_estudiants_en_practiques = desti.total_estudiants_en_practiques + 1
                      where desti.any_universitari_id = any_universitari and professor_nif = :NEW.professor_nif
            when not matched then insert (desti.any_universitari_id, professor_nif, total_estudiants_en_practiques)
                                         values(any_universitari, :NEW.professor_nif, 1);

            /** total d'estudiants en practiques per titulacio **/
            merge into e_titulacio_dades desti
            using dual
                on ( desti.titulacio_id = p_titulacio_id)
            when matched then update 
                            set desti.total_estudiants_en_practiques_seleccionats = desti.total_estudiants_en_practiques_seleccionats + 1
                      where desti.titulacio_id = p_titulacio_id
            when not matched then insert (desti.titulacio_id, desti.total_estudiants_en_practiques_seleccionats)
                                         values(p_titulacio_id, 1);

            
             /** sou total d'estudiants en practiques per universitat i any universitari **/
            update e_estudiants_per_universitat_i_any_universitari
            set sou_total = sou_total + :NEW.sou,
                    dies_total_contracte = dies_total_contracte + extract( day from ((:NEW.data_fi - :NEW.data_inici))),
                    total_contractes = total_contractes+ 1 
            where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
            
            /**sou total d'estudiants en practiques per any universitari **/
            update e_estudiants_any_universitari
            set sou_total = sou_total + :NEW.sou,
                dies_total_contracte = dies_total_contracte + extract( day from ((:NEW.data_fi - :NEW.data_inici))),
                    total_contractes = total_contractes+ 1 
            where any_universitari_id = any_universitari;

            /**sou total d'estudiants en practiques per any universitari **/
            update e_estudiants_dades
            set sou_total = sou_total + :NEW.sou,
                dies_total_contracte = dies_total_contracte + extract( day from ((:NEW.data_fi - :NEW.data_inici))),
                    total_contractes = total_contractes+ 1 
            where id = 1;
            
            /**sou total d'estudiants en practiques per universitat **/
            update e_estudiants_per_universitat
            set sou_total = sou_total + :NEW.sou,
                dies_total_contracte = dies_total_contracte + extract( day from ((:NEW.data_fi - :NEW.data_inici))),
                    total_contractes = total_contractes+ 1 
            where universitat_id = p_universitat_id ;

        /* actualitzem els estudiants que superen les practiques */ 
        when updating then
        
            if(:NEW.sou <> :OLD.sou)then
                 update e_estudiants_per_universitat_i_any_universitari
                set sou_total = sou_total + (:NEW.sou - :OLD.sou)
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
            
                /**sou total d'estudiants en practiques per any universitari **/
                update e_estudiants_any_universitari
                set sou_total = sou_total + (:NEW.sou - :OLD.sou)
                where any_universitari_id = any_universitari;
    
                /**sou total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set sou_total = sou_total + (:NEW.sou - :OLD.sou)
                where id = 1;
                
                /**sou total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set sou_total = sou_total + (:NEW.sou - :OLD.sou)
                where universitat_id = p_universitat_id ;
            
            end if;
            
            if(:NEW.data_inici <> :OLD.data_inici or :NEW.data_fi <> :OLD.data_fi)then
            
                update e_estudiants_per_universitat_i_any_universitari
                set dies_total_contracte = dies_total_contracte - extract( day from ((:OLD.data_fi - :OLD.data_inici))) + extract( day from ((:NEW.data_fi - :NEW.data_inici)))
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
            
                update e_estudiants_any_universitari
                set dies_total_contracte = dies_total_contracte - extract( day from ((:OLD.data_fi - :OLD.data_inici))) + extract( day from ((:NEW.data_fi - :NEW.data_inici)))
                where any_universitari_id = any_universitari;
    
                /**sou total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set dies_total_contracte = dies_total_contracte - extract( day from ((:OLD.data_fi - :OLD.data_inici))) + extract( day from ((:NEW.data_fi - :NEW.data_inici)))
                where id = 1;
                
                /**sou total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set dies_total_contracte = dies_total_contracte - extract( day from ((:OLD.data_fi - :OLD.data_inici))) + extract( day from ((:NEW.data_fi - :NEW.data_inici)))
                where universitat_id = p_universitat_id ;
            
            end if;

            if(:NEW.es_apte = 1 and :OLD.es_apte = 0) then
                /** totals d'estudiants en practiques per universitat **/
                update e_professor_i_any_universitari
                set total_estudiants_aprovats = total_estudiants_aprovats + 1
                where any_universitari_id = any_universitari and professor_nif = :NEW.professor_nif;

                /** total d'estudiants en practiques per titulacio **/
                update e_titulacio_dades
                set total_estudiants_en_practiques_finalitzat_satisfactori = total_estudiants_en_practiques_finalitzat_satisfactori + 1
                where titulacio_id = p_titulacio_id;

                /** total d'estudiants en practiques per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_aprovats = estudiants_aprovats + 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_aprovats = estudiants_aprovats + 1
                where any_universitari_id = any_universitari;

                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set estudiants_aprovats = estudiants_aprovats + 1
                where id = 1;
                
                /** total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set estudiants_aprovats = estudiants_aprovats + 1
                where universitat_id = p_universitat_id ;

            elsif (:NEW.es_apte = 0 and :OLD.es_apte = 1) then
                 /** totals d'estudiants en practiques per universitat **/
                update e_professor_i_any_universitari
                set total_estudiants_aprovats = total_estudiants_aprovats - 1
                where any_universitari_id = any_universitari and professor_nif = :NEW.professor_nif;

                /** total d'estudiants en practiques per titulacio **/
                update e_titulacio_dades
                set total_estudiants_en_practiques_finalitzat_satisfactori = total_estudiants_en_practiques_finalitzat_satisfactori - 1
                where titulacio_id = p_titulacio_id;
                
                /** total d'estudiants en practiques per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_aprovats = estudiants_aprovats - 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_aprovats = estudiants_aprovats - 1
                where any_universitari_id = any_universitari;

                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set estudiants_aprovats = estudiants_aprovats - 1
                where id = 1;                
                
                /** total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set estudiants_aprovats = estudiants_aprovats - 1
                where universitat_id = p_universitat_id;
            end if;  
        
            if(:NEW.data_informe is not null and :OLD.data_informe is null) then
                /** total d'estudiants en practiques per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_finalitzats = estudiants_finalitzats + 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_finalitzats = estudiants_finalitzats + 1
                where any_universitari_id = any_universitari;

                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set estudiants_finalitzats = estudiants_finalitzats + 1
                where id = 1;
                
                /** total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set estudiants_finalitzats = estudiants_finalitzats + 1
                where universitat_id = p_universitat_id ;
            elsif (:NEW.data_informe is null and :OLD.data_informe is not null) then
                /** total d'estudiants en practiques per universitat i any universitari **/
                update e_estudiants_per_universitat_i_any_universitari
                set estudiants_finalitzats = estudiants_finalitzats - 1
                where universitat_id = p_universitat_id and any_universitari_id = any_universitari;
                
                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_any_universitari
                set estudiants_finalitzats = estudiants_finalitzats - 1
                where any_universitari_id = any_universitari;

                /** total d'estudiants en practiques per any universitari **/
                update e_estudiants_dades
                set estudiants_finalitzats = estudiants_finalitzats - 1
                where id = 1;
                
                /** total d'estudiants en practiques per universitat **/
                update e_estudiants_per_universitat
                set estudiants_finalitzats = estudiants_finalitzats - 1
                where universitat_id = p_universitat_id ;
                
            end if;
            

    end case;

end;

/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula contracte_practiques
**/

create or replace NONEDITIONABLE TRIGGER E_CONTRACTE_PRACTIQUES_DELETE 
BEFORE DELETE ON contracte_practiques
    for each row
declare
    /** es necessari per poder capturar quan s'elimina una dada que, mitjanant
    l'integritat referencial de la dada, fa que s'elimini la dada de candidat. 
    Per exemple, quan s'elimina una oferta de treball **/
    pragma autonomous_transaction;

    any_universitari number;
    p_universitat_id number;
    p_titulacio_id number;
    p_data timestamp;

    RSP varchar2(2000);
BEGIN
    select universitat_id, titulacio_id, data_creacio
        into p_universitat_id, p_titulacio_id, p_data
    from oferta_treball
    where id = :OLD.oferta_treball_id;

    any_universitari :=  buscaAnyUniversitari( p_data => p_data);

    /** totals d'estudiants en practiques per universitat **/
    begin
        merge into e_professor_i_any_universitari desti
        using dual
            on ( any_universitari_id = any_universitari and desti.professor_nif = :OLD.professor_nif )
        when matched then update set desti.total_estudiants_aprovats = desti.total_estudiants_aprovats - 1
                  where any_universitari_id = any_universitari and desti.professor_nif = :OLD.professor_nif;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /** total d'estudiants en practiques per titulacio **/
    begin
        merge into e_titulacio_dades desti
        using dual
            on ( desti.titulacio_id = p_titulacio_id )
        when matched then update set desti.total_estudiants_en_practiques_finalitzat_satisfactori = desti.total_estudiants_en_practiques_finalitzat_satisfactori - 1
                  where desti.titulacio_id = p_titulacio_id;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;
    
    

        /** total d'estudiants en practiques per universitat i any universitari **/
        begin
            merge into e_estudiants_per_universitat_i_any_universitari desti
            using dual
                on ( desti.universitat_id = p_universitat_id and desti.any_universitari_id = any_universitari )
            when matched then update set desti.total_contractes = desti.total_contractes - 1,
                                        dies_total_contracte = dies_total_contracte + extract( day from ((:OLD.data_fi - :OLD.data_inici))),
                                        sou_total = sou_total - :OLD.sou
                      where desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total d'estudiants en practiques per any universitari **/
        begin
            merge into e_estudiants_any_universitari desti
            using dual
                on ( desti.any_universitari_id = any_universitari )
            when matched then update set desti.total_contractes = desti.total_contractes - 1,
                                        dies_total_contracte = dies_total_contracte + extract( day from ((:OLD.data_fi - :OLD.data_inici))),
                                        sou_total = sou_total - :OLD.sou
                      where any_universitari_id = any_universitari ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total d'estudiants en practiques per any universitari **/
        begin
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1 )
            when matched then update set desti.total_contractes = desti.total_contractes - 1,
                                        dies_total_contracte = dies_total_contracte + extract( day from ((:OLD.data_fi - :OLD.data_inici))),
                                        sou_total = sou_total - :OLD.sou
                      where id = 1 ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        

        /** total d'estudiants en practiques per universitat **/
        begin
            merge into e_estudiants_per_universitat desti
            using dual
                on ( desti.universitat_id = p_universitat_id )
            when matched then update set desti.total_contractes = desti.total_contractes - 1,
                                        dies_total_contracte = dies_total_contracte + extract( day from ((:OLD.data_fi - :OLD.data_inici))),
                                        sou_total = sou_total - :OLD.sou
                      where desti.universitat_id = p_universitat_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
    
    

    if (:OLD.es_apte = 1) then
         /** totals d'estudiants en practiques per universitat **/
        begin
            merge into e_professor_i_any_universitari desti
            using dual
                on ( any_universitari_id = any_universitari and desti.professor_nif = :OLD.professor_nif )
            when matched then update set desti.total_estudiants_aprovats = desti.total_estudiants_aprovats - 1
                      where any_universitari_id = any_universitari and desti.professor_nif = :OLD.professor_nif;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

        /** total d'estudiants en practiques per titulacio **/

        begin
            merge into e_titulacio_dades desti
            using dual
                on ( desti.titulacio_id = p_titulacio_id )
            when matched then update set desti.total_estudiants_en_practiques_finalitzat_satisfactori = desti.total_estudiants_en_practiques_finalitzat_satisfactori - 1
                      where desti.titulacio_id = p_titulacio_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

        /** total d'estudiants en practiques per universitat i any universitari **/
        begin
            merge into e_estudiants_per_universitat_i_any_universitari desti
            using dual
                on ( desti.universitat_id = p_universitat_id and desti.any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_aprovats = desti.estudiants_aprovats - 1
                      where desti.universitat_id = p_universitat_id and any_universitari_id = any_universitari ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total d'estudiants en practiques per any universitari **/
        begin
            merge into e_estudiants_any_universitari desti
            using dual
                on ( desti.any_universitari_id = any_universitari )
            when matched then update set desti.estudiants_aprovats = desti.estudiants_aprovats - 1
                      where any_universitari_id = any_universitari ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        /** total d'estudiants en practiques per any universitari **/
        begin
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1 )
            when matched then update set desti.estudiants_aprovats = desti.estudiants_aprovats - 1
                      where id = 1 ;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;
        
        

        /** total d'estudiants en practiques per universitat **/
        begin
            merge into e_estudiants_per_universitat desti
            using dual
                on ( desti.universitat_id = p_universitat_id )
            when matched then update set desti.estudiants_aprovats = desti.estudiants_aprovats - 1
                      where desti.universitat_id = p_universitat_id;
        exception
            when no_data_found then
                 RSP := 'ERROR|NO S''HAN TROBAT DADES';
        end;

    end if;  
    commit;
END;


/
/**
Disparador que s'executa quan s'afegeixen 
dades  a la taula inspeccio_treball:
**/
create or replace NONEDITIONABLE trigger e_inspeccio_treball 
after insert on inspeccio_treball
for each row
declare
    p_companyia_cif char(9);
    p_data timestamp;
    any_universitari number;
    
    RSP varchar2(2000);
begin

    select companyia_cif, data_creacio
        into p_companyia_cif, p_data
    from oferta_treball
    where id in (
        select oferta_treball_id 
        from contracte_practiques
        where id = :NEW.contracte_practiques_id
    );
        
    any_universitari :=  buscaAnyUniversitari( p_data => p_data );
    
     begin

        merge into e_companyia_dades desti
        using dual
            on ( desti.companyia_cif = p_companyia_cif )
        when matched then update set desti.total_inspeccions_rebudes = total_inspeccions_rebudes + 1
                  where desti.companyia_cif = p_companyia_cif
        when not matched then insert (desti.companyia_cif, total_inspeccions_rebudes)
                                 values(p_companyia_cif, 1);
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;
    
    begin

        merge into e_companyia_dades_i_any_universitari desti
        using dual
            on ( desti.companyia_cif = p_companyia_cif and any_universitari_id = any_universitari )
        when matched then update set desti.total_inspeccions_rebudes = total_inspeccions_rebudes + 1
                  where desti.companyia_cif = p_companyia_cif  and any_universitari_id = any_universitari
        when not matched then insert (desti.companyia_cif, any_universitari_id, total_inspeccions_rebudes)
                                 values(p_companyia_cif, any_universitari, 1);
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

end;

/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula inspeccio_treball:
**/

create or replace NONEDITIONABLE TRIGGER E_INSPECCIO_TREBALL_DELETE 
after delete on inspeccio_treball
for each row
declare
    /** es necessari per poder capturar quan s'elimina una dada que, mitjanant
    l'integritat referencial de la dada, fa que s'elimini la dada de candidat. 
    Per exemple, quan s'elimina una oferta de treball **/
    pragma autonomous_transaction;
    p_companyia_cif char(9);
    p_data timestamp;
    any_universitari number;
    
    RSP varchar2(2000);
begin

    
    select companyia_cif, data_creacio
        into p_companyia_cif, p_data
    from oferta_treball
    where id in (
        select oferta_treball_id 
        from contracte_practiques
        where id = :OLD.id
    );
    
    any_universitari :=  buscaAnyUniversitari( p_data => p_data );

    begin
        merge into e_companyia_dades desti
        using dual
            on ( desti.companyia_cif = p_companyia_cif )
        when matched then update set desti.total_inspeccions_rebudes = total_inspeccions_rebudes - 1
                  where desti.companyia_cif = p_companyia_cif;
    exception
        when no_data_found then
            RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    begin
        merge into e_companyia_dades_i_any_universitari desti
        using dual
            on ( desti.companyia_cif = p_companyia_cif and any_universitari_id = any_universitari  )
        when matched then update set desti.total_inspeccions_rebudes = total_inspeccions_rebudes - 1
                  where desti.companyia_cif = p_companyia_cif and any_universitari_id = any_universitari;
    exception
        when no_data_found then
            RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;
    commit;
    
exception
        when no_data_found then
            RSP := 'ERROR|NO S''HAN TROBAT DADES';
end;



/
/**
Disparador que s'executa quan s'afegeixen 
dades a la taula oferta_treball:
**/

create or replace NONEDITIONABLE trigger e_oferta_treball
    after insert on oferta_treball

    for each row
declare
    any_universitari number;
begin

    any_universitari :=  buscaAnyUniversitari( p_data => :NEW.data_creacio );

    -- ofertes de treball per companyia i any universitari
    merge into e_companyia_dades_i_any_universitari desti
    using dual
        on ( desti.companyia_cif = :NEW.companyia_cif and any_universitari_id = any_universitari )
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where desti.companyia_cif = :NEW.companyia_cif and any_universitari_id = any_universitari
    when not matched then insert (desti.companyia_cif, any_universitari_id, total_ofertes_treball)
                                 values(:NEW.companyia_cif, any_universitari, 1);

    -- ofertes de treball per companyia
    merge into e_companyia_dades desti
    using dual
        on ( desti.companyia_cif = :NEW.companyia_cif )
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where desti.companyia_cif = :NEW.companyia_cif
    when not matched then insert (desti.companyia_cif, total_ofertes_treball)
                                 values(:NEW.companyia_cif, 1);

    -- total ofertes de treball per universitat i any universitari
    merge into e_estudiants_per_universitat_i_any_universitari desti
    using dual
        on ( desti.universitat_id = :NEW.universitat_id and desti.any_universitari_id = any_universitari)
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where desti.universitat_id = :NEW.universitat_id and desti.any_universitari_id = any_universitari
    when not matched then insert (desti.universitat_id, desti.any_universitari_id, desti.total_ofertes_treball)
                        values(:NEW.universitat_id, any_universitari, 1);
                        
     -- total ofertes de treball per universitat i any universitari
    merge into e_estudiants_any_universitari desti
    using dual
        on ( desti.any_universitari_id = any_universitari)
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where desti.any_universitari_id = any_universitari
    when not matched then insert (desti.any_universitari_id, desti.total_ofertes_treball)
                        values( any_universitari, 1);   
                        
    -- total ofertes de treball per universitat i any universitari
    merge into e_estudiants_dades desti
    using dual
        on ( id = 1)
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where id = 1
    when not matched then insert (desti.id,  desti.total_ofertes_treball)
                        values( 1, 1);
    

     -- total ofertes de treball per universitat 
    merge into e_estudiants_per_universitat desti
    using dual
        on ( desti.universitat_id = :NEW.universitat_id )
    when matched then update set desti.total_ofertes_treball = total_ofertes_treball + 1
              where desti.universitat_id = :NEW.universitat_id
    when not matched then insert (desti.universitat_id, desti.total_ofertes_treball)
                        values(:NEW.universitat_id, 1);

end;

/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula oferta_treball
**/

create or replace NONEDITIONABLE TRIGGER E_OFERTA_TREBALL_DELETE 
BEFORE DELETE ON oferta_treball
    for each row
declare
    /** es necessari per poder capturar quan s'elimina una dada que, mitjanant
    l'integritat referencial de la dada, fa que s'elimini la dada de candidat. 
    Per exemple, quan s'elimina una oferta de treball **/
    pragma autonomous_transaction;

    any_universitari number;
    RSP varchar2(2000);
BEGIN

    any_universitari :=  buscaAnyUniversitari( p_data => :OLD.data_creacio );

    /**ofertes de treball per companyia i any universitari*/
    begin
        merge into e_companyia_dades_i_any_universitari desti
        using dual
            on ( desti.companyia_cif = :OLD.companyia_cif and desti.any_universitari_id = any_universitari )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where desti.companyia_cif = :OLD.companyia_cif and desti.any_universitari_id = any_universitari;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /**ofertes de treball per companyia;**/
    begin
        merge into e_companyia_dades desti
        using dual
            on ( desti.companyia_cif = :OLD.companyia_cif  )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where desti.companyia_cif = :OLD.companyia_cif ;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /** total ofertes de treball per universitat i any universitari*/
     begin
        merge into e_estudiants_per_universitat_i_any_universitari desti
        using dual
            on ( desti.universitat_id = :OLD.universitat_id and desti.any_universitari_id = any_universitari )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where desti.universitat_id = :OLD.universitat_id and desti.any_universitari_id = any_universitari;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /** total ofertes de treball per any universitari*/
     begin
        merge into e_estudiants_any_universitari desti
        using dual
            on ( desti.any_universitari_id = any_universitari )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where  desti.any_universitari_id = any_universitari;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /** total ofertes de treball per any universitari*/
     begin
        merge into e_estudiants_dades desti
        using dual
            on ( id = 1 )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where  id = 1;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    /** total ofertes de treball per universitat */
    begin
        merge into e_estudiants_per_universitat desti
        using dual
            on ( desti.universitat_id = :OLD.universitat_id  )
        when matched then update set desti.total_ofertes_treball = total_ofertes_treball - 1
                  where desti.universitat_id = :OLD.universitat_id ;
    exception
        when no_data_found then
             RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;

    commit;
END;


/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula professor
**/

create or replace NONEDITIONABLE TRIGGER E_PROFESSOR_DELETE 
after DELETE ON professor
for each row
BEGIN
    delete from e_professor_i_any_universitari where professor_nif = :OLD.nif ;
END;



/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula titulacio
**/
create or replace NONEDITIONABLE TRIGGER E_TITULACIO_DELETE 
after DELETE ON titulacio
for each row
declare
    
    RSP varchar2(2000);
BEGIN
    begin
    
        delete from e_titulacio_dades where titulacio_id = :OLD.id ;
    exception
        when no_data_found then
            RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;
    
   
END;

/
/**
Disparador que s'executa quan s'eliminen 
dades  a la taula universitat
**/
create or replace NONEDITIONABLE TRIGGER E_UNIVERSITAT_DELETE 
after DELETE ON universitat
for each row
BEGIN
    delete from e_estudiants_per_universitat_i_any_universitari where universitat_id = :OLD.id ;
    delete from e_estudiants_per_universitat where universitat_id = :OLD.id ;
END;

/
CREATE OR REPLACE TRIGGER E_ESTUDIANT 
after INSERT OR UPDATE of universitat_id ON estudiant 

for each row
declare
    test number;
begin
    case 
        when inserting then
        select id into test
        from universitat
        where id = :new.universitat_id;
            /** totals d'estudiants en practiques per universitat **/
            merge into e_estudiants_dades desti
            using dual
                on ( id = 1)
            when matched then update 
                            set desti.total_estudiants = total_estudiants + 1
                      where id = 1
            when not matched then insert (id, total_estudiants)
                                         values(1, 1); 
                                         
             /** totals d'estudiants en practiques per universitat **/
            merge into e_estudiants_per_universitat desti
            using dual
                on ( universitat_id = :NEW.universitat_id)
            when matched then update 
                            set desti.total_estudiants = total_estudiants + 1
                      where universitat_id = :NEW.universitat_id
            when not matched then insert (universitat_id, total_estudiants)
                                         values(  :NEW.universitat_id, 1);
        when updating then
            if(:NEW.universitat_id <> :OLD.universitat_id) then
                
                update e_estudiants_per_universitat
                set total_estudiants = total_estudiants -1
                where universitat_id = :OLD.universitat_id;
                
                merge into e_estudiants_per_universitat desti
                using dual
                    on ( universitat_id = :NEW.universitat_id)
                when matched then update 
                                set desti.total_estudiants = total_estudiants + 1
                          where universitat_id = :NEW.universitat_id
                when not matched then insert (universitat_id, total_estudiants)
                                             values( :NEW.universitat_id, 1);
            end if;

    end case;
END;

/
CREATE OR REPLACE TRIGGER E_ESTUDIANT_DELETE 
after DELETE ON estudiant
for each row
declare
    RSP varchar2(2000);
BEGIN
    begin

        update e_estudiants_dades
                set total_estudiants = total_estudiants -1
                where id = 1;
        update e_estudiants_per_universitat
                set total_estudiants = total_estudiants -1
                where universitat_id = :OLD.universitat_id;
    exception
        when no_data_found then
            RSP := 'ERROR|NO S''HAN TROBAT DADES';
    end;


END;